@@ -29,7 +29,7 @@ module Hledger.Data.AccountName (
   ,expandAccountName
   ,expandAccountNames
   ,isAccountNamePrefixOf
---  ,isAccountRegex 
+--  ,isAccountRegex
   ,isSubAccountNameOf
   ,parentAccountName
   ,parentAccountNames
@@ -41,6 +41,7 @@ module Hledger.Data.AccountName (
 where
 
 import Data.List
+import Data.List.Extra (nubSort)
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
 #endif
@@ -50,7 +51,7 @@ import Data.Tree
 import Text.Printf
 
 import Hledger.Data.Types
-import Hledger.Utils 
+import Hledger.Utils
 
 -- $setup
 -- >>> :set -XOverloadedStrings
@@ -88,13 +89,13 @@ accountNameLevel "" = 0
 accountNameLevel a = T.length (T.filter (==acctsepchar) a) + 1
 
 -- | A top-level account prefixed to some accounts in budget reports.
--- Defined here so it can be ignored by accountNameDrop. 
+-- Defined here so it can be ignored by accountNameDrop.
 unbudgetedAccountName :: T.Text
 unbudgetedAccountName = "<unbudgeted>"
 
 -- | Remove some number of account name components from the front of the account name.
 -- If the special "<unbudgeted>" top-level account is present, it is preserved and
--- dropping affects the rest of the account name. 
+-- dropping affects the rest of the account name.
 accountNameDrop :: Int -> AccountName -> AccountName
 accountNameDrop n a
   | a == unbudgetedAccountName = a
@@ -103,14 +104,14 @@ accountNameDrop n a
         "" -> unbudgetedAccountName
         a' -> unbudgetedAccountAndSep <> a'
   | otherwise = accountNameFromComponents $ drop n $ accountNameComponents a
-  where 
+  where
     unbudgetedAccountAndSep = unbudgetedAccountName <> acctsep
 
 -- | Sorted unique account names implied by these account names,
 -- ie these plus all their parent accounts up to the root.
 -- Eg: ["a:b:c","d:e"] -> ["a","a:b","a:b:c","d","d:e"]
 expandAccountNames :: [AccountName] -> [AccountName]
-expandAccountNames as = nub $ sort $ concatMap expandAccountName as
+expandAccountNames as = nubSort $ concatMap expandAccountName as
 
 -- | "a:b:c" -> ["a","a:b","a:b:c"]
 expandAccountName :: AccountName -> [AccountName]
@@ -227,27 +228,23 @@ accountRegexToAccountName = T.pack . regexReplace "^\\^(.*?)\\(:\\|\\$\\)$" "\\1
 --isAccountRegex s = take 1 s == "^" && take 5 (reverse s) == ")$|:("
 
 tests_AccountName = tests "AccountName" [
-  tests "accountNameTreeFrom" [
-     accountNameTreeFrom ["a"]       `is` Node "root" [Node "a" []]
-    ,accountNameTreeFrom ["a","b"]   `is` Node "root" [Node "a" [], Node "b" []]
-    ,accountNameTreeFrom ["a","a:b"] `is` Node "root" [Node "a" [Node "a:b" []]]
-    ,accountNameTreeFrom ["a:b:c"]   `is` Node "root" [Node "a" [Node "a:b" [Node "a:b:c" []]]]
-  ]
-  ,tests "expandAccountNames" [
-    expandAccountNames ["assets:cash","assets:checking","expenses:vacation"] `is`
+   test "accountNameTreeFrom" $ do
+    accountNameTreeFrom ["a"]       @?= Node "root" [Node "a" []]
+    accountNameTreeFrom ["a","b"]   @?= Node "root" [Node "a" [], Node "b" []]
+    accountNameTreeFrom ["a","a:b"] @?= Node "root" [Node "a" [Node "a:b" []]]
+    accountNameTreeFrom ["a:b:c"]   @?= Node "root" [Node "a" [Node "a:b" [Node "a:b:c" []]]]
+  ,test "expandAccountNames" $ do
+    expandAccountNames ["assets:cash","assets:checking","expenses:vacation"] @?=
      ["assets","assets:cash","assets:checking","expenses","expenses:vacation"]
-  ]
-  ,tests "isAccountNamePrefixOf" [
-     "assets" `isAccountNamePrefixOf` "assets" `is` False
-    ,"assets" `isAccountNamePrefixOf` "assets:bank" `is` True
-    ,"assets" `isAccountNamePrefixOf` "assets:bank:checking" `is` True
-    ,"my assets" `isAccountNamePrefixOf` "assets:bank" `is` False
-  ]
-  ,tests "isSubAccountNameOf" [
-     "assets" `isSubAccountNameOf` "assets" `is` False
-    ,"assets:bank" `isSubAccountNameOf` "assets" `is` True
-    ,"assets:bank:checking" `isSubAccountNameOf` "assets" `is` False
-    ,"assets:bank" `isSubAccountNameOf` "my assets" `is` False
-  ]
+  ,test "isAccountNamePrefixOf" $ do
+    "assets" `isAccountNamePrefixOf` "assets" @?= False
+    "assets" `isAccountNamePrefixOf` "assets:bank" @?= True
+    "assets" `isAccountNamePrefixOf` "assets:bank:checking" @?= True
+    "my assets" `isAccountNamePrefixOf` "assets:bank" @?= False
+  ,test "isSubAccountNameOf" $ do
+    "assets" `isSubAccountNameOf` "assets" @?= False
+    "assets:bank" `isSubAccountNameOf` "assets" @?= True
+    "assets:bank:checking" `isSubAccountNameOf` "assets" @?= False
+    "assets:bank" `isSubAccountNameOf` "my assets" @?= False
  ]
 
