@@ -40,6 +40,10 @@ exchange rates.
 
 -}
 
+-- Silence safe 0.3.18's deprecation warnings for (max|min)imum(By)?Def for now
+-- (may hide other deprecation warnings too). https://github.com/ndmitchell/safe/issues/26
+{-# OPTIONS_GHC -Wno-warnings-deprecations #-}
+
 {-# LANGUAGE StandaloneDeriving, RecordWildCards, OverloadedStrings #-}
 
 module Hledger.Data.Amount (
@@ -51,21 +55,24 @@ module Hledger.Data.Amount (
   usd,
   eur,
   gbp,
+  per,
   hrs,
   at,
   (@@),
   amountWithCommodity,
   -- ** arithmetic
-  costOfAmount,
+  amountCost,
+  amountIsZero,
+  amountLooksZero,
   divideAmount,
   multiplyAmount,
   divideAmountAndPrice,
   multiplyAmountAndPrice,
-  amountValue,
   amountTotalPriceToUnitPrice,
   -- ** rendering
   amountstyle,
   styleAmount,
+  styleAmountExceptPrecision,
   showAmount,
   cshowAmount,
   showAmountWithZeroCommodity,
@@ -76,7 +83,8 @@ module Hledger.Data.Amount (
   setAmountPrecision,
   withPrecision,
   setFullPrecision,
-  setMinimalPrecision,
+  setNaturalPrecision,
+  setNaturalPrecisionUpTo,
   setAmountInternalPrecision,
   withInternalPrecision,
   setAmountDecimalPoint,
@@ -89,10 +97,12 @@ module Hledger.Data.Amount (
   amounts,
   filterMixedAmount,
   filterMixedAmountByCommodity,
+  mapMixedAmount,
   normaliseMixedAmountSquashPricesForDisplay,
   normaliseMixedAmount,
+  mixedAmountStripPrices,
   -- ** arithmetic
-  costOfMixedAmount,
+  mixedAmountCost,
   divideMixedAmount,
   multiplyMixedAmount,
   divideMixedAmountAndPrice,
@@ -100,12 +110,8 @@ module Hledger.Data.Amount (
   averageMixedAmounts,
   isNegativeAmount,
   isNegativeMixedAmount,
-  isZeroAmount,
-  isReallyZeroAmount,
-  isZeroMixedAmount,
-  isReallyZeroMixedAmount,
-  isReallyZeroMixedAmountCost,
-  mixedAmountValue,
+  mixedAmountIsZero,
+  mixedAmountLooksZero,
   mixedAmountTotalPriceToUnitPrice,
   -- ** rendering
   styleMixedAmount,
@@ -129,18 +135,16 @@ import Data.Char (isDigit)
 import Data.Decimal (roundTo, decimalPlaces, normalizeDecimal)
 import Data.Function (on)
 import Data.List
+import qualified Data.Map as M
 import Data.Map (findWithDefault)
 import Data.Maybe
-import Data.Time.Calendar (Day)
--- import Data.Text (Text)
 import qualified Data.Text as T
 import Safe (maximumDef)
 import Text.Printf
-import qualified Data.Map as M
 
 import Hledger.Data.Types
 import Hledger.Data.Commodity
-import Hledger.Utils 
+import Hledger.Utils
 
 
 deriving instance Show MarketPrice
@@ -149,7 +153,7 @@ deriving instance Show MarketPrice
 -------------------------------------------------------------------------------
 -- Amount styles
 
--- | Default amount style 
+-- | Default amount style
 amountstyle = AmountStyle L False 0 (Just '.') Nothing
 
 
@@ -167,7 +171,7 @@ instance Num Amount where
 
 -- | The empty simple amount.
 amount, nullamt :: Amount
-amount = Amount{acommodity="", aquantity=0, aprice=NoPrice, astyle=amountstyle, aismultiplier=False}
+amount = Amount{acommodity="", aquantity=0, aprice=Nothing, astyle=amountstyle, aismultiplier=False}
 nullamt = amount
 
 -- | A temporary value for parsed transactions which had no amount specified.
@@ -181,8 +185,9 @@ hrs n = amount{acommodity="h", aquantity=n,           astyle=amountstyle{aspreci
 usd n = amount{acommodity="$", aquantity=roundTo 2 n, astyle=amountstyle{asprecision=2}}
 eur n = amount{acommodity="€", aquantity=roundTo 2 n, astyle=amountstyle{asprecision=2}}
 gbp n = amount{acommodity="£", aquantity=roundTo 2 n, astyle=amountstyle{asprecision=2}}
-amt `at` priceamt = amt{aprice=UnitPrice priceamt}
-amt @@ priceamt = amt{aprice=TotalPrice priceamt}
+per n = amount{acommodity="%", aquantity=n,           astyle=amountstyle{asprecision=1, ascommodityside=R, ascommodityspaced=True}}
+amt `at` priceamt = amt{aprice=Just $ UnitPrice priceamt}
+amt @@ priceamt = amt{aprice=Just $ TotalPrice priceamt}
 
 -- | Apply a binary arithmetic operator to two amounts, which should
 -- be in the same commodity if non-zero (warning, this is not checked).
@@ -202,29 +207,32 @@ similarAmountsOp op Amount{acommodity=_,  aquantity=q1, astyle=AmountStyle{aspre
 -- | Convert an amount to the specified commodity, ignoring and discarding
 -- any assigned prices and assuming an exchange rate of 1.
 amountWithCommodity :: CommoditySymbol -> Amount -> Amount
-amountWithCommodity c a = a{acommodity=c, aprice=NoPrice}
+amountWithCommodity c a = a{acommodity=c, aprice=Nothing}
 
--- | Convert an amount to the commodity of its assigned price, if any.  Notes:
+-- | Convert a amount to its "cost" or "selling price" in another commodity,
+-- using its attached transaction price if it has one.  Notes:
 --
--- - price amounts must be MixedAmounts with exactly one component Amount (or there will be a runtime error) XXX
+-- - price amounts must be MixedAmounts with exactly one component Amount
+--   (or there will be a runtime error XXX)
 --
--- - price amounts should be positive, though this is not currently enforced
-costOfAmount :: Amount -> Amount
-costOfAmount a@Amount{aquantity=q, aprice=price} =
-    case price of
-      NoPrice -> a
-      UnitPrice  p@Amount{aquantity=pq} -> p{aquantity=pq * q}
-      TotalPrice p@Amount{aquantity=pq} -> p{aquantity=pq * signum q}
+-- - price amounts should be positive
+--   (though this is currently not enforced)
+amountCost :: Amount -> Amount
+amountCost a@Amount{aquantity=q, aprice=mp} =
+    case mp of
+      Nothing                                  -> a
+      Just (UnitPrice  p@Amount{aquantity=pq}) -> p{aquantity=pq * q}
+      Just (TotalPrice p@Amount{aquantity=pq}) -> p{aquantity=pq * signum q}
 
 -- | Replace an amount's TotalPrice, if it has one, with an equivalent UnitPrice.
 -- Has no effect on amounts without one.
 -- Also increases the unit price's display precision to show one extra decimal place,
--- to help keep transaction amounts balancing. 
+-- to help keep transaction amounts balancing.
 -- Does Decimal division, might be some rounding/irrational number issues.
 amountTotalPriceToUnitPrice :: Amount -> Amount
-amountTotalPriceToUnitPrice 
-  a@Amount{aquantity=q, aprice=TotalPrice pa@Amount{aquantity=pq, astyle=ps@AmountStyle{asprecision=pp}}}
-  = a{aprice = UnitPrice pa{aquantity=abs (pq/q), astyle=ps{asprecision=pp+1}}}
+amountTotalPriceToUnitPrice
+  a@Amount{aquantity=q, aprice=Just (TotalPrice pa@Amount{aquantity=pq, astyle=ps@AmountStyle{asprecision=pp}})}
+  = a{aprice = Just $ UnitPrice pa{aquantity=abs (pq/q), astyle=ps{asprecision=pp+1}}}
 amountTotalPriceToUnitPrice a = a
 
 -- | Divide an amount's quantity by a constant.
@@ -238,7 +246,7 @@ multiplyAmount n a@Amount{aquantity=q} = a{aquantity=q*n}
 -- | Divide an amount's quantity (and its total price, if it has one) by a constant.
 -- The total price will be kept positive regardless of the multiplier's sign.
 divideAmountAndPrice :: Quantity -> Amount -> Amount
-divideAmountAndPrice n a@Amount{aquantity=q,aprice=p} = a{aquantity=q/n, aprice=f p}
+divideAmountAndPrice n a@Amount{aquantity=q,aprice=p} = a{aquantity=q/n, aprice=f <$> p}
   where
     f (TotalPrice a) = TotalPrice $ abs $ n `divideAmount` a
     f p = p
@@ -246,7 +254,7 @@ divideAmountAndPrice n a@Amount{aquantity=q,aprice=p} = a{aquantity=q/n, aprice=
 -- | Multiply an amount's quantity (and its total price, if it has one) by a constant.
 -- The total price will be kept positive regardless of the multiplier's sign.
 multiplyAmountAndPrice :: Quantity -> Amount -> Amount
-multiplyAmountAndPrice n a@Amount{aquantity=q,aprice=p} = a{aquantity=q*n, aprice=f p}
+multiplyAmountAndPrice n a@Amount{aquantity=q,aprice=p} = a{aquantity=q*n, aprice=f <$> p}
   where
     f (TotalPrice a) = TotalPrice $ abs $ n `multiplyAmount` a
     f p = p
@@ -257,42 +265,52 @@ isNegativeAmount Amount{aquantity=q} = q < 0
 
 digits = "123456789" :: String
 
--- | Does this amount appear to be zero when displayed with its given precision ?
-isZeroAmount :: Amount -> Bool
-isZeroAmount --  a==missingamt = False
-  = not . any (`elem` digits) . showAmountWithoutPriceOrCommodity
+-- | Does mixed amount appear to be zero when rendered with its
+-- display precision ?
+amountLooksZero :: Amount -> Bool
+amountLooksZero = not . any (`elem` digits) . showAmountWithoutPriceOrCommodity
 
--- | Is this amount "really" zero, regardless of the display precision ?
-isReallyZeroAmount :: Amount -> Bool
-isReallyZeroAmount Amount{aquantity=q} = q == 0
+-- | Is this amount exactly zero, ignoring its display precision ?
+amountIsZero :: Amount -> Bool
+amountIsZero Amount{aquantity=q} = q == 0
 
 -- | Get the string representation of an amount, based on its commodity's
 -- display settings except using the specified precision.
 showAmountWithPrecision :: Int -> Amount -> String
 showAmountWithPrecision p = showAmount . setAmountPrecision p
 
--- | Set an amount's display precision.
-setAmountPrecision :: Int -> Amount -> Amount
-setAmountPrecision p a@Amount{astyle=s} = a{astyle=s{asprecision=p}}
-
 -- | Set an amount's display precision, flipped.
 withPrecision :: Amount -> Int -> Amount
 withPrecision = flip setAmountPrecision
 
--- | Increase an amount's display precision, if necessary, enough so
--- that it will be shown exactly, with all significant decimal places
--- (excluding trailing zeros).
+-- | Set an amount's display precision.
+setAmountPrecision :: Int -> Amount -> Amount
+setAmountPrecision p a@Amount{astyle=s} = a{astyle=s{asprecision=p}}
+
+-- | Increase an amount's display precision, if needed, to enough
+-- decimal places to show it exactly (showing all significant decimal
+-- digits, excluding trailing zeros).
 setFullPrecision :: Amount -> Amount
 setFullPrecision a = setAmountPrecision p a
   where
-    p                = max displayprecision normalprecision
+    p                = max displayprecision naturalprecision
     displayprecision = asprecision $ astyle a
+    naturalprecision = fromIntegral $ decimalPlaces $ normalizeDecimal $ aquantity a
+
+-- | Set an amount's display precision to just enough decimal places
+-- to show it exactly (possibly less than the number specified by
+-- the amount's display style).
+setNaturalPrecision :: Amount -> Amount
+setNaturalPrecision a = setAmountPrecision normalprecision a
+  where
     normalprecision  = fromIntegral $ decimalPlaces $ normalizeDecimal $ aquantity a
 
--- | Set an amount's display precision to just enough so that it will
--- be shown exactly, with all significant decimal places.
-setMinimalPrecision :: Amount -> Amount
-setMinimalPrecision a = setAmountPrecision normalprecision a
+-- | Set an amount's display precision to just enough decimal places
+-- to show it exactly (possibly less than the number specified by the
+-- amount's display style), but not more than the given maximum number
+-- of decimal digits.
+setNaturalPrecisionUpTo :: Int -> Amount -> Amount
+setNaturalPrecisionUpTo n a = setAmountPrecision (min n normalprecision) a
   where
     normalprecision  = fromIntegral $ decimalPlaces $ normalizeDecimal $ aquantity a
 
@@ -300,26 +318,26 @@ setMinimalPrecision a = setAmountPrecision normalprecision a
 -- appropriate to the current debug level. 9 shows maximum detail.
 showAmountDebug :: Amount -> String
 showAmountDebug Amount{acommodity="AUTO"} = "(missing)"
-showAmountDebug Amount{..} = printf "Amount {acommodity=%s, aquantity=%s, aprice=%s, astyle=%s}" (show acommodity) (show aquantity) (showPriceDebug aprice) (show astyle)
+showAmountDebug Amount{..} = printf "Amount {acommodity=%s, aquantity=%s, aprice=%s, astyle=%s}" (show acommodity) (show aquantity) (showAmountPriceDebug aprice) (show astyle)
 
 -- | Get the string representation of an amount, without any \@ price.
 showAmountWithoutPrice :: Amount -> String
-showAmountWithoutPrice a = showAmount a{aprice=NoPrice}
+showAmountWithoutPrice a = showAmount a{aprice=Nothing}
 
--- | Set an amount's internal precision, ie rounds the Decimal representing 
+-- | Set an amount's internal precision, ie rounds the Decimal representing
 -- the amount's quantity to some number of decimal places.
 -- Rounding is done with Data.Decimal's default roundTo function:
 -- "If the value ends in 5 then it is rounded to the nearest even value (Banker's Rounding)".
 -- Does not change the amount's display precision.
--- Intended only for internal use, eg when comparing amounts in tests. 
+-- Intended only for internal use, eg when comparing amounts in tests.
 setAmountInternalPrecision :: Int -> Amount -> Amount
-setAmountInternalPrecision p a@Amount{ aquantity=q, astyle=s } = a{ 
-   astyle=s{asprecision=p} 
+setAmountInternalPrecision p a@Amount{ aquantity=q, astyle=s } = a{
+   astyle=s{asprecision=p}
   ,aquantity=roundTo (fromIntegral p) q
   }
 
 -- | Set an amount's internal precision, flipped.
--- Intended only for internal use, eg when comparing amounts in tests. 
+-- Intended only for internal use, eg when comparing amounts in tests.
 withInternalPrecision :: Amount -> Int -> Amount
 withInternalPrecision = flip setAmountInternalPrecision
 
@@ -333,29 +351,37 @@ withDecimalPoint = flip setAmountDecimalPoint
 
 -- | Colour version.
 cshowAmountWithoutPrice :: Amount -> String
-cshowAmountWithoutPrice a = cshowAmount a{aprice=NoPrice}
+cshowAmountWithoutPrice a = cshowAmount a{aprice=Nothing}
 
 -- | Get the string representation of an amount, without any price or commodity symbol.
 showAmountWithoutPriceOrCommodity :: Amount -> String
-showAmountWithoutPriceOrCommodity a = showAmount a{acommodity="", aprice=NoPrice}
+showAmountWithoutPriceOrCommodity a = showAmount a{acommodity="", aprice=Nothing}
 
-showPrice :: Price -> String
-showPrice NoPrice         = ""
-showPrice (UnitPrice pa)  = " @ "  ++ showAmount pa
-showPrice (TotalPrice pa) = " @@ " ++ showAmount pa
+showAmountPrice :: Maybe AmountPrice -> String
+showAmountPrice Nothing                = ""
+showAmountPrice (Just (UnitPrice pa))  = " @ "  ++ showAmount pa
+showAmountPrice (Just (TotalPrice pa)) = " @@ " ++ showAmount pa
 
-showPriceDebug :: Price -> String
-showPriceDebug NoPrice         = ""
-showPriceDebug (UnitPrice pa)  = " @ "  ++ showAmountDebug pa
-showPriceDebug (TotalPrice pa) = " @@ " ++ showAmountDebug pa
+showAmountPriceDebug :: Maybe AmountPrice -> String
+showAmountPriceDebug Nothing                = ""
+showAmountPriceDebug (Just (UnitPrice pa))  = " @ "  ++ showAmountDebug pa
+showAmountPriceDebug (Just (TotalPrice pa)) = " @@ " ++ showAmountDebug pa
 
--- | Given a map of standard amount display styles, apply the appropriate one to this amount.
--- If there's no standard style for this amount's commodity, return the amount unchanged.
+-- | Given a map of standard commodity display styles, apply the
+-- appropriate one to this amount. If there's no standard style for
+-- this amount's commodity, return the amount unchanged.
 styleAmount :: M.Map CommoditySymbol AmountStyle -> Amount -> Amount
 styleAmount styles a =
   case M.lookup (acommodity a) styles of
     Just s  -> a{astyle=s}
-    Nothing -> a 
+    Nothing -> a
+
+-- | Like styleAmount, but keep the number of decimal places unchanged.
+styleAmountExceptPrecision :: M.Map CommoditySymbol AmountStyle -> Amount -> Amount
+styleAmountExceptPrecision styles a@Amount{astyle=AmountStyle{asprecision=origp}} =
+  case M.lookup (acommodity a) styles of
+    Just s  -> a{astyle=s{asprecision=origp}}
+    Nothing -> a
 
 -- | Get the string representation of an amount, based on its
 -- commodity's display settings. String representations equivalent to
@@ -364,7 +390,7 @@ styleAmount styles a =
 showAmount :: Amount -> String
 showAmount = showAmountHelper False
 
--- | Colour version. For a negative amount, adds ANSI codes to change the colour, 
+-- | Colour version. For a negative amount, adds ANSI codes to change the colour,
 -- currently to hard-coded red.
 cshowAmount :: Amount -> String
 cshowAmount a =
@@ -373,7 +399,7 @@ cshowAmount a =
 
 showAmountHelper :: Bool -> Amount -> String
 showAmountHelper _ Amount{acommodity="AUTO"} = ""
-showAmountHelper showzerocommodity a@Amount{acommodity=c, aprice=p, astyle=AmountStyle{..}} =
+showAmountHelper showzerocommodity a@Amount{acommodity=c, aprice=mp, astyle=AmountStyle{..}} =
     case ascommodityside of
       L -> printf "%s%s%s%s" (T.unpack c') space quantity' price
       R -> printf "%s%s%s%s" quantity' space (T.unpack c') price
@@ -383,7 +409,7 @@ showAmountHelper showzerocommodity a@Amount{acommodity=c, aprice=p, astyle=Amoun
       (quantity',c') | displayingzero && not showzerocommodity = ("0","")
                      | otherwise = (quantity, quoteCommoditySymbolIfNeeded c)
       space = if not (T.null c') && ascommodityspaced then " " else "" :: String
-      price = showPrice p
+      price = showAmountPrice mp
 
 -- | Like showAmount, but show a zero amount's commodity if it has one.
 showAmountWithZeroCommodity :: Amount -> String
@@ -401,9 +427,9 @@ showamountquantity Amount{aquantity=q, astyle=AmountStyle{asprecision=p, asdecim
         | p == maxprecision          = chopdotzero $ show q
         | otherwise                  = show $ roundTo (fromIntegral p) q
 
--- | Replace a number string's decimal point with the specified character,
--- and add the specified digit group separators. The last digit group will
--- be repeated as needed.
+-- | Replace a number string's decimal mark with the specified
+-- character, and add the specified digit group marks. The last digit
+-- group will be repeated as needed.
 punctuatenumber :: Char -> Maybe DigitGroupStyle -> String -> String
 punctuatenumber dec mgrps s = sign ++ reverse (applyDigitGroupStyle mgrps (reverse int)) ++ frac''
     where
@@ -444,36 +470,6 @@ canonicaliseAmount styles a@Amount{acommodity=c, astyle=s} = a{astyle=s'}
     where
       s' = findWithDefault s c styles
 
--- | Find the market value of this amount on the given date, in it's
--- default valuation commodity, based on recorded market prices.
--- If no default valuation commodity can be found, the amount is left
--- unchanged.
-amountValue :: Journal -> Day -> Amount -> Amount
-amountValue j d a =
-  case commodityValue j d (acommodity a) of
-    Just v  -> v{aquantity=aquantity v * aquantity a}
-    Nothing -> a
-
--- This is here not in Commodity.hs to use the Amount Show instance above for debugging. 
--- | Find the market value, if known, of one unit of this commodity (A) on
--- the given valuation date, in the commodity (B) mentioned in the latest
--- applicable market price. The latest applicable market price is the market
--- price directive for commodity A with the latest date that is on or before
--- the valuation date; or if there are multiple such prices with the same date,
--- the last parsed.
-commodityValue :: Journal -> Day -> CommoditySymbol -> Maybe Amount
-commodityValue j valuationdate c
-    | null applicableprices = dbg Nothing
-    | otherwise             = dbg $ Just $ mpamount $ last applicableprices
-  where
-    dbg = dbg8 ("using market price for "++T.unpack c)
-    applicableprices =
-      [p | p <- sortOn mpdate $ jmarketprices j
-      , mpcommodity p == c
-      , mpdate p <= valuationdate
-      ]
-
-
 -------------------------------------------------------------------------------
 -- MixedAmount
 
@@ -521,7 +517,7 @@ normaliseHelper squashprices (Mixed as)
     newzero = case filter (/= "") (map acommodity zeros) of
                _:_ -> last zeros
                _   -> nullamt
-    (zeros, nonzeros) = partition isReallyZeroAmount $
+    (zeros, nonzeros) = partition amountIsZero $
                         map sumSimilarAmountsUsingFirstPrice $
                         groupBy groupfn $
                         sortBy sortfn
@@ -531,8 +527,8 @@ normaliseHelper squashprices (Mixed as)
     groupfn | squashprices = (==) `on` acommodity
             | otherwise    = \a1 a2 -> acommodity a1 == acommodity a2 && combinableprices a1 a2
 
-    combinableprices Amount{aprice=NoPrice} Amount{aprice=NoPrice} = True
-    combinableprices Amount{aprice=UnitPrice p1} Amount{aprice=UnitPrice p2} = p1 == p2
+    combinableprices Amount{aprice=Nothing} Amount{aprice=Nothing} = True
+    combinableprices Amount{aprice=Just (UnitPrice p1)} Amount{aprice=Just (UnitPrice p2)} = p1 == p2
     combinableprices _ _ = False
 
 -- | Like normaliseMixedAmount, but combine each commodity's amounts
@@ -578,10 +574,10 @@ filterMixedAmountByCommodity c (Mixed as) = Mixed as'
 mapMixedAmount :: (Amount -> Amount) -> MixedAmount -> MixedAmount
 mapMixedAmount f (Mixed as) = Mixed $ map f as
 
--- | Convert a mixed amount's component amounts to the commodity of their
--- assigned price, if any.
-costOfMixedAmount :: MixedAmount -> MixedAmount
-costOfMixedAmount (Mixed as) = Mixed $ map costOfAmount as
+-- | Convert all component amounts to cost/selling price where
+-- possible (see amountCost).
+mixedAmountCost :: MixedAmount -> MixedAmount
+mixedAmountCost (Mixed as) = Mixed $ map amountCost as
 
 -- | Divide a mixed amount's quantities by a constant.
 divideMixedAmount :: Quantity -> MixedAmount -> MixedAmount
@@ -604,38 +600,40 @@ multiplyMixedAmountAndPrice n = mapMixedAmount (multiplyAmountAndPrice n)
 -- | Calculate the average of some mixed amounts.
 averageMixedAmounts :: [MixedAmount] -> MixedAmount
 averageMixedAmounts [] = 0
-averageMixedAmounts as = fromIntegral (length as) `divideMixedAmount` sum as 
+averageMixedAmounts as = fromIntegral (length as) `divideMixedAmount` sum as
 
--- | Is this mixed amount negative, if it can be normalised to a single commodity ?
+-- | Is this mixed amount negative, if we can tell that unambiguously?
+-- Ie when normalised, are all individual commodity amounts negative ?
 isNegativeMixedAmount :: MixedAmount -> Maybe Bool
-isNegativeMixedAmount m = case as of [a] -> Just $ isNegativeAmount a
-                                     _   -> Nothing
-    where as = amounts $ normaliseMixedAmountSquashPricesForDisplay m
+isNegativeMixedAmount m =
+  case amounts $ normaliseMixedAmountSquashPricesForDisplay m of
+    []  -> Just False
+    [a] -> Just $ isNegativeAmount a
+    as | all isNegativeAmount as -> Just True
+    as | not (any isNegativeAmount as) -> Just False
+    _ -> Nothing  -- multiple amounts with different signs
 
--- | Does this mixed amount appear to be zero when displayed with its given precision ?
-isZeroMixedAmount :: MixedAmount -> Bool
-isZeroMixedAmount = all isZeroAmount . amounts . normaliseMixedAmountSquashPricesForDisplay
+-- | Does this mixed amount appear to be zero when rendered with its
+-- display precision ?
+mixedAmountLooksZero :: MixedAmount -> Bool
+mixedAmountLooksZero = all amountLooksZero . amounts . normaliseMixedAmountSquashPricesForDisplay
 
--- | Is this mixed amount "really" zero ? See isReallyZeroAmount.
-isReallyZeroMixedAmount :: MixedAmount -> Bool
-isReallyZeroMixedAmount = all isReallyZeroAmount . amounts . normaliseMixedAmountSquashPricesForDisplay
-
--- | Is this mixed amount "really" zero, after converting to cost
--- commodities where possible ?
-isReallyZeroMixedAmountCost :: MixedAmount -> Bool
-isReallyZeroMixedAmountCost = isReallyZeroMixedAmount . costOfMixedAmount
+-- | Is this mixed amount exactly zero, ignoring display precisions ?
+mixedAmountIsZero :: MixedAmount -> Bool
+mixedAmountIsZero = all amountIsZero . amounts . normaliseMixedAmountSquashPricesForDisplay
 
 -- -- | MixedAmount derived Eq instance in Types.hs doesn't know that we
 -- -- want $0 = EUR0 = 0. Yet we don't want to drag all this code over there.
 -- -- For now, use this when cross-commodity zero equality is important.
 -- mixedAmountEquals :: MixedAmount -> MixedAmount -> Bool
--- mixedAmountEquals a b = amounts a' == amounts b' || (isZeroMixedAmount a' && isZeroMixedAmount b')
+-- mixedAmountEquals a b = amounts a' == amounts b' || (mixedAmountLooksZero a' && mixedAmountLooksZero b')
 --     where a' = normaliseMixedAmountSquashPricesForDisplay a
 --           b' = normaliseMixedAmountSquashPricesForDisplay b
 
--- | Given a map of standard amount display styles, apply the appropriate ones to each individual amount.
+-- | Given a map of standard commodity display styles, apply the
+-- appropriate one to each individual amount.
 styleMixedAmount :: M.Map CommoditySymbol AmountStyle -> MixedAmount -> MixedAmount
-styleMixedAmount styles (Mixed as) = Mixed $ map (styleAmount styles) as 
+styleMixedAmount styles (Mixed as) = Mixed $ map (styleAmount styles) as
 
 -- | Get the string representation of a mixed amount, after
 -- normalising it to one amount per commodity. Assumes amounts have
@@ -706,7 +704,7 @@ cshowMixedAmountWithoutPrice m = intercalate "\n" $ map showamt as
         width = maximumDef 0 $ map (length . showAmount) as
 
 mixedAmountStripPrices :: MixedAmount -> MixedAmount
-mixedAmountStripPrices (Mixed as) = Mixed $ map (\a -> a{aprice=NoPrice}) as
+mixedAmountStripPrices (Mixed as) = Mixed $ map (\a -> a{aprice=Nothing}) as
 
 -- | Get the one-line string representation of a mixed amount, but without
 -- any \@ prices.
@@ -714,24 +712,21 @@ showMixedAmountOneLineWithoutPrice :: MixedAmount -> String
 showMixedAmountOneLineWithoutPrice m = intercalate ", " $ map showAmountWithoutPrice as
     where
       (Mixed as) = normaliseMixedAmountSquashPricesForDisplay $ stripPrices m
-      stripPrices (Mixed as) = Mixed $ map stripprice as where stripprice a = a{aprice=NoPrice}
+      stripPrices (Mixed as) = Mixed $ map stripprice as where stripprice a = a{aprice=Nothing}
 
 -- | Colour version.
 cshowMixedAmountOneLineWithoutPrice :: MixedAmount -> String
 cshowMixedAmountOneLineWithoutPrice m = intercalate ", " $ map cshowAmountWithoutPrice as
     where
       (Mixed as) = normaliseMixedAmountSquashPricesForDisplay $ stripPrices m
-      stripPrices (Mixed as) = Mixed $ map stripprice as where stripprice a = a{aprice=NoPrice}
+      stripPrices (Mixed as) = Mixed $ map stripprice as where stripprice a = a{aprice=Nothing}
 
 -- | Canonicalise a mixed amount's display styles using the provided commodity style map.
 canonicaliseMixedAmount :: M.Map CommoditySymbol AmountStyle -> MixedAmount -> MixedAmount
 canonicaliseMixedAmount styles (Mixed as) = Mixed $ map (canonicaliseAmount styles) as
 
-mixedAmountValue :: Journal -> Day -> MixedAmount -> MixedAmount
-mixedAmountValue j d (Mixed as) = Mixed $ map (amountValue j d) as
-
 -- | Replace each component amount's TotalPrice, if it has one, with an equivalent UnitPrice.
--- Has no effect on amounts without one. 
+-- Has no effect on amounts without one.
 -- Does Decimal division, might be some rounding/irrational number issues.
 mixedAmountTotalPriceToUnitPrice :: MixedAmount -> MixedAmount
 mixedAmountTotalPriceToUnitPrice (Mixed as) = Mixed $ map amountTotalPriceToUnitPrice as
@@ -743,99 +738,88 @@ mixedAmountTotalPriceToUnitPrice (Mixed as) = Mixed $ map amountTotalPriceToUnit
 tests_Amount = tests "Amount" [
    tests "Amount" [
 
-     tests "costOfAmount" [
-       costOfAmount (eur 1) `is` eur 1
-      ,costOfAmount (eur 2){aprice=UnitPrice $ usd 2} `is` usd 4
-      ,costOfAmount (eur 1){aprice=TotalPrice $ usd 2} `is` usd 2
-      ,costOfAmount (eur (-1)){aprice=TotalPrice $ usd 2} `is` usd (-2)
-    ]
-  
-    ,tests "isZeroAmount" [
-       expect $ isZeroAmount amount
-      ,expect $ isZeroAmount $ usd 0
-    ]
-  
-    ,tests "negating amounts" [
-       negate (usd 1) `is` (usd 1){aquantity= -1}
-      ,let b = (usd 1){aprice=UnitPrice $ eur 2} in negate b `is` b{aquantity= -1}
-    ]
-  
-    ,tests "adding amounts without prices" [
-       (usd 1.23 + usd (-1.23)) `is` usd 0
-      ,(usd 1.23 + usd (-1.23)) `is` usd 0
-      ,(usd (-1.23) + usd (-1.23)) `is` usd (-2.46)
-      ,sum [usd 1.23,usd (-1.23),usd (-1.23),-(usd (-1.23))] `is` usd 0
-      -- highest precision is preserved
-      ,asprecision (astyle $ sum [usd 1 `withPrecision` 1, usd 1 `withPrecision` 3]) `is` 3
-      ,asprecision (astyle $ sum [usd 1 `withPrecision` 3, usd 1 `withPrecision` 1]) `is` 3
-      -- adding different commodities assumes conversion rate 1
-      ,expect $ isZeroAmount (usd 1.23 - eur 1.23)
-    ]
-  
-    ,tests "showAmount" [
-      showAmount (usd 0 + gbp 0) `is` "0"
-    ]
+     test "amountCost" $ do
+       amountCost (eur 1) @?= eur 1
+       amountCost (eur 2){aprice=Just $ UnitPrice $ usd 2} @?= usd 4
+       amountCost (eur 1){aprice=Just $ TotalPrice $ usd 2} @?= usd 2
+       amountCost (eur (-1)){aprice=Just $ TotalPrice $ usd 2} @?= usd (-2)
+
+    ,test "amountLooksZero" $ do
+       assertBool "" $ amountLooksZero amount
+       assertBool "" $ amountLooksZero $ usd 0
+
+    ,test "negating amounts" $ do
+       negate (usd 1) @?= (usd 1){aquantity= -1}
+       let b = (usd 1){aprice=Just $ UnitPrice $ eur 2} in negate b @?= b{aquantity= -1}
+
+    ,test "adding amounts without prices" $ do
+       (usd 1.23 + usd (-1.23)) @?= usd 0
+       (usd 1.23 + usd (-1.23)) @?= usd 0
+       (usd (-1.23) + usd (-1.23)) @?= usd (-2.46)
+       sum [usd 1.23,usd (-1.23),usd (-1.23),-(usd (-1.23))] @?= usd 0
+       -- highest precision is preserved
+       asprecision (astyle $ sum [usd 1 `withPrecision` 1, usd 1 `withPrecision` 3]) @?= 3
+       asprecision (astyle $ sum [usd 1 `withPrecision` 3, usd 1 `withPrecision` 1]) @?= 3
+       -- adding different commodities assumes conversion rate 1
+       assertBool "" $ amountLooksZero (usd 1.23 - eur 1.23)
+
+    ,test "showAmount" $ do
+      showAmount (usd 0 + gbp 0) @?= "0"
 
   ]
 
   ,tests "MixedAmount" [
 
-     tests "adding mixed amounts to zero, the commodity and amount style are preserved" [
+     test "adding mixed amounts to zero, the commodity and amount style are preserved" $
       sum (map (Mixed . (:[]))
                [usd 1.25
                ,usd (-1) `withPrecision` 3
                ,usd (-0.25)
                ])
-        `is` Mixed [usd 0 `withPrecision` 3]
-    ]
-  
-    ,tests "adding mixed amounts with total prices" [
+        @?= Mixed [usd 0 `withPrecision` 3]
+
+    ,test "adding mixed amounts with total prices" $ do
       sum (map (Mixed . (:[]))
        [usd 1 @@ eur 1
        ,usd (-2) @@ eur 1
        ])
-        `is` Mixed [usd 1 @@ eur 1
+        @?= Mixed [usd 1 @@ eur 1
                    ,usd (-2) @@ eur 1
                    ]
-    ]
-  
-    ,tests "showMixedAmount" [
-       showMixedAmount (Mixed [usd 1]) `is` "$1.00"
-      ,showMixedAmount (Mixed [usd 1 `at` eur 2]) `is` "$1.00 @ €2.00"
-      ,showMixedAmount (Mixed [usd 0]) `is` "0"
-      ,showMixedAmount (Mixed []) `is` "0"
-      ,showMixedAmount missingmixedamt `is` ""
-    ]
-  
-    ,tests "showMixedAmountWithoutPrice" $
-      let a = usd 1 `at` eur 2 in 
-    [
-        showMixedAmountWithoutPrice (Mixed [a]) `is` "$1.00"
-       ,showMixedAmountWithoutPrice (Mixed [a, -a]) `is` "0"
-    ]
-  
+
+    ,test "showMixedAmount" $ do
+       showMixedAmount (Mixed [usd 1]) @?= "$1.00"
+       showMixedAmount (Mixed [usd 1 `at` eur 2]) @?= "$1.00 @ €2.00"
+       showMixedAmount (Mixed [usd 0]) @?= "0"
+       showMixedAmount (Mixed []) @?= "0"
+       showMixedAmount missingmixedamt @?= ""
+
+    ,test "showMixedAmountWithoutPrice" $ do
+      let a = usd 1 `at` eur 2
+      showMixedAmountWithoutPrice (Mixed [a]) @?= "$1.00"
+      showMixedAmountWithoutPrice (Mixed [a, -a]) @?= "0"
+
     ,tests "normaliseMixedAmount" [
-       test "a missing amount overrides any other amounts" $ 
-        normaliseMixedAmount (Mixed [usd 1, missingamt]) `is` missingmixedamt
-      ,test "unpriced same-commodity amounts are combined" $ 
-        normaliseMixedAmount (Mixed [usd 0, usd 2]) `is` Mixed [usd 2]
-      ,test "amounts with same unit price are combined" $ 
-        normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 1]) `is` Mixed [usd 2 `at` eur 1]
-      ,test "amounts with different unit prices are not combined" $ 
-        normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]) `is` Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]
+       test "a missing amount overrides any other amounts" $
+        normaliseMixedAmount (Mixed [usd 1, missingamt]) @?= missingmixedamt
+      ,test "unpriced same-commodity amounts are combined" $
+        normaliseMixedAmount (Mixed [usd 0, usd 2]) @?= Mixed [usd 2]
+      ,test "amounts with same unit price are combined" $
+        normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 1]) @?= Mixed [usd 2 `at` eur 1]
+      ,test "amounts with different unit prices are not combined" $
+        normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]) @?= Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]
       ,test "amounts with total prices are not combined" $
-        normaliseMixedAmount (Mixed  [usd 1 @@ eur 1, usd 1 @@ eur 1]) `is` Mixed [usd 1 @@ eur 1, usd 1 @@ eur 1]
+        normaliseMixedAmount (Mixed  [usd 1 @@ eur 1, usd 1 @@ eur 1]) @?= Mixed [usd 1 @@ eur 1, usd 1 @@ eur 1]
     ]
-  
-    ,tests "normaliseMixedAmountSquashPricesForDisplay" [
-       normaliseMixedAmountSquashPricesForDisplay (Mixed []) `is` Mixed [nullamt]
-      ,expect $ isZeroMixedAmount $ normaliseMixedAmountSquashPricesForDisplay
+
+    ,test "normaliseMixedAmountSquashPricesForDisplay" $ do
+       normaliseMixedAmountSquashPricesForDisplay (Mixed []) @?= Mixed [nullamt]
+       assertBool "" $ mixedAmountLooksZero $ normaliseMixedAmountSquashPricesForDisplay
         (Mixed [usd 10
                ,usd 10 @@ eur 7
                ,usd (-10)
                ,usd (-10) @@ eur 7
                ])
-    ]
 
   ]
 
