@@ -60,10 +60,11 @@ module Hledger.Data.Dates (
   spansSpan,
   spanIntersect,
   spansIntersect,
-  spanIntervalIntersect,
   spanDefaultsFrom,
   spanUnion,
   spansUnion,
+  daysSpan,
+  latestSpanContaining,
   smartdate,
   splitSpan,
   fixSmartDate,
@@ -77,12 +78,15 @@ module Hledger.Data.Dates (
 where
 
 import Prelude ()
-import "base-compat-batteries" Prelude.Compat
+import "base-compat-batteries" Prelude.Compat hiding (fail)
+import qualified "base-compat-batteries" Control.Monad.Fail.Compat as Fail (MonadFail, fail)
 import Control.Applicative.Permutations
-import Control.Monad
+import Control.Monad (guard, unless)
 import "base-compat-batteries" Data.List.Compat
 import Data.Default
+import Data.Foldable (asum)
 import Data.Maybe
+import qualified Data.Set as Set
 import Data.Text (Text)
 import qualified Data.Text as T
 #if MIN_VERSION_time(1,5,0)
@@ -95,7 +99,7 @@ import Data.Time.Calendar
 import Data.Time.Calendar.OrdinalDate
 import Data.Time.Clock
 import Data.Time.LocalTime
-import Safe (headMay, lastMay, readMay)
+import Safe (headMay, lastMay, readMay, maximumMay, minimumMay)
 import Text.Megaparsec
 import Text.Megaparsec.Char
 import Text.Megaparsec.Custom
@@ -112,7 +116,7 @@ instance Show DateSpan where
     -- show s = "DateSpan \"" ++ showDateSpan s ++ "\"" -- quotes to help pretty-show
 
 showDate :: Day -> String
-showDate = formatTime defaultTimeLocale "%0C%y/%m/%d"
+showDate = show
 
 -- | Render a datespan as a display string, abbreviating into a
 -- compact form if possible.
@@ -156,38 +160,38 @@ spansSpan spans = DateSpan (maybe Nothing spanStart $ headMay spans) (maybe Noth
 -- If no interval is specified, the original span is returned.
 -- If the original span is the null date span, ie unbounded, the null date span is returned.
 -- If the original span is empty, eg if the end date is <= the start date, no spans are returned.
--- 
+--
 --
 -- ==== Examples:
 -- >>> let t i d1 d2 = splitSpan i $ mkdatespan d1 d2
 -- >>> t NoInterval "2008/01/01" "2009/01/01"
 -- [DateSpan 2008]
 -- >>> t (Quarters 1) "2008/01/01" "2009/01/01"
--- [DateSpan 2008q1,DateSpan 2008q2,DateSpan 2008q3,DateSpan 2008q4]
+-- [DateSpan 2008Q1,DateSpan 2008Q2,DateSpan 2008Q3,DateSpan 2008Q4]
 -- >>> splitSpan (Quarters 1) nulldatespan
--- [DateSpan -]
+-- [DateSpan ..]
 -- >>> t (Days 1) "2008/01/01" "2008/01/01"  -- an empty datespan
 -- []
 -- >>> t (Quarters 1) "2008/01/01" "2008/01/01"
 -- []
 -- >>> t (Months 1) "2008/01/01" "2008/04/01"
--- [DateSpan 2008/01,DateSpan 2008/02,DateSpan 2008/03]
+-- [DateSpan 2008-01,DateSpan 2008-02,DateSpan 2008-03]
 -- >>> t (Months 2) "2008/01/01" "2008/04/01"
--- [DateSpan 2008/01/01-2008/02/29,DateSpan 2008/03/01-2008/04/30]
+-- [DateSpan 2008-01-01..2008-02-29,DateSpan 2008-03-01..2008-04-30]
 -- >>> t (Weeks 1) "2008/01/01" "2008/01/15"
--- [DateSpan 2007/12/31w01,DateSpan 2008/01/07w02,DateSpan 2008/01/14w03]
+-- [DateSpan 2007-12-31W01,DateSpan 2008-01-07W02,DateSpan 2008-01-14W03]
 -- >>> t (Weeks 2) "2008/01/01" "2008/01/15"
--- [DateSpan 2007/12/31-2008/01/13,DateSpan 2008/01/14-2008/01/27]
+-- [DateSpan 2007-12-31..2008-01-13,DateSpan 2008-01-14..2008-01-27]
 -- >>> t (DayOfMonth 2) "2008/01/01" "2008/04/01"
--- [DateSpan 2007/12/02-2008/01/01,DateSpan 2008/01/02-2008/02/01,DateSpan 2008/02/02-2008/03/01,DateSpan 2008/03/02-2008/04/01]
+-- [DateSpan 2007-12-02..2008-01-01,DateSpan 2008-01-02..2008-02-01,DateSpan 2008-02-02..2008-03-01,DateSpan 2008-03-02..2008-04-01]
 -- >>> t (WeekdayOfMonth 2 4) "2011/01/01" "2011/02/15"
--- [DateSpan 2010/12/09-2011/01/12,DateSpan 2011/01/13-2011/02/09,DateSpan 2011/02/10-2011/03/09]
+-- [DateSpan 2010-12-09..2011-01-12,DateSpan 2011-01-13..2011-02-09,DateSpan 2011-02-10..2011-03-09]
 -- >>> t (DayOfWeek 2) "2011/01/01" "2011/01/15"
--- [DateSpan 2010/12/28-2011/01/03,DateSpan 2011/01/04-2011/01/10,DateSpan 2011/01/11-2011/01/17]
+-- [DateSpan 2010-12-28..2011-01-03,DateSpan 2011-01-04..2011-01-10,DateSpan 2011-01-11..2011-01-17]
 -- >>> t (DayOfYear 11 29) "2011/10/01" "2011/10/15"
--- [DateSpan 2010/11/29-2011/11/28]
+-- [DateSpan 2010-11-29..2011-11-28]
 -- >>> t (DayOfYear 11 29) "2011/12/01" "2012/12/15"
--- [DateSpan 2011/11/29-2012/11/28,DateSpan 2012/11/29-2013/11/28]
+-- [DateSpan 2011-11-29..2012-11-28,DateSpan 2012-11-29..2013-11-28]
 --
 splitSpan :: Interval -> DateSpan -> [DateSpan]
 splitSpan _ (DateSpan Nothing Nothing) = [DateSpan Nothing Nothing]
@@ -231,9 +235,8 @@ daysInSpan _ = Nothing
 
 -- | Is this an empty span, ie closed with the end date on or before the start date ?
 isEmptySpan :: DateSpan -> Bool
-isEmptySpan s = case daysInSpan s of
-                  Just n  -> n < 1
-                  Nothing -> False
+isEmptySpan (DateSpan (Just s) (Just e)) = e <= s
+isEmptySpan _                            = False
 
 -- | Does the span include the given date ?
 spanContainsDate :: DateSpan -> Day -> Bool
@@ -256,33 +259,12 @@ spansIntersect (d:ds) = d `spanIntersect` (spansIntersect ds)
 --
 -- For non-intersecting spans, gives an empty span beginning on the second's start date:
 -- >>> mkdatespan "2018-01-01" "2018-01-03" `spanIntersect` mkdatespan "2018-01-03" "2018-01-05"
--- DateSpan 2018/01/03-2018/01/02
+-- DateSpan 2018-01-03..2018-01-02
 spanIntersect (DateSpan b1 e1) (DateSpan b2 e2) = DateSpan b e
     where
       b = latest b1 b2
       e = earliest e1 e2
 
--- | Calculate the intersection of two DateSpans, adjusting the start date so
--- the interval is preserved.
---
--- >>> let intervalIntersect = spanIntervalIntersect (Days 3)
--- >>> mkdatespan "2018-01-01" "2018-01-03" `intervalIntersect` mkdatespan "2018-01-01" "2018-01-05"
--- DateSpan 2018/01/01-2018/01/02
--- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2018-01-02" "2018-01-05"
--- DateSpan 2018/01/04
--- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2018-01-03" "2018-01-05"
--- DateSpan 2018/01/04
--- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2018-01-04" "2018-01-05"
--- DateSpan 2018/01/04
--- >>> mkdatespan "2018-01-01" "2018-01-05" `intervalIntersect` mkdatespan "2017-12-01" "2018-01-05"
--- DateSpan 2018/01/01-2018/01/04
-spanIntervalIntersect :: Interval -> DateSpan -> DateSpan -> DateSpan
-spanIntervalIntersect (Days n) (DateSpan (Just b1) e1) sp2@(DateSpan (Just b2) _) =
-      DateSpan (Just b) e1 `spanIntersect` sp2
-    where
-      b = if b1 < b2 then addDays (diffDays b1 b2 `mod` toInteger n) b2 else b1
-spanIntervalIntersect _ sp1 sp2 = sp1 `spanIntersect` sp2
-
 -- | Fill any unspecified dates in the first span with the dates from
 -- the second one. Sort of a one-way spanIntersect.
 spanDefaultsFrom (DateSpan a1 b1) (DateSpan a2 b2) = DateSpan a b
@@ -308,6 +290,36 @@ earliest d Nothing = d
 earliest Nothing d = d
 earliest (Just d1) (Just d2) = Just $ min d1 d2
 
+-- | Calculate the minimal DateSpan containing all of the given Days (in the
+-- usual exclusive-end-date sense: beginning on the earliest, and ending on
+-- the day after the latest).
+daysSpan :: [Day] -> DateSpan
+daysSpan ds = DateSpan (minimumMay ds) (addDays 1 <$> maximumMay ds)
+
+-- | Select the DateSpan containing a given Day, if any, from a given list of
+-- DateSpans.
+--
+-- If the DateSpans are non-overlapping, this returns the unique containing
+-- DateSpan, if it exists. If the DateSpans are overlapping, it will return the
+-- containing DateSpan with the latest start date, and then latest end date.
+
+-- Note: This will currently return `DateSpan (Just s) (Just e)` before it will
+-- return `DateSpan (Just s) Nothing`. It's unclear which behaviour is desired.
+-- This is irrelevant at the moment as it's never applied to any list with
+-- overlapping DateSpans.
+latestSpanContaining :: [DateSpan] -> Day -> Maybe DateSpan
+latestSpanContaining datespans = go
+  where
+    go day = do
+        span <- Set.lookupLT supSpan spanSet
+        guard $ spanContainsDate span day
+        return span
+      where
+        -- The smallest DateSpan larger than any DateSpan containing day.
+        supSpan = DateSpan (Just $ addDays 1 day) Nothing
+
+    spanSet = Set.fromList $ filter (not . isEmptySpan) datespans
+
 -- | Parse a period expression to an Interval and overall DateSpan using
 -- the provided reference date, or return a parse error.
 parsePeriodExpr
@@ -394,72 +406,72 @@ fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
 -- >>> :set -XOverloadedStrings
 -- >>> let t = fixSmartDateStr (parsedate "2008/11/26")
 -- >>> t "0000-01-01"
--- "0000/01/01"
+-- "0000-01-01"
 -- >>> t "1999-12-02"
--- "1999/12/02"
+-- "1999-12-02"
 -- >>> t "1999.12.02"
--- "1999/12/02"
+-- "1999-12-02"
 -- >>> t "1999/3/2"
--- "1999/03/02"
+-- "1999-03-02"
 -- >>> t "19990302"
--- "1999/03/02"
+-- "1999-03-02"
 -- >>> t "2008/2"
--- "2008/02/01"
+-- "2008-02-01"
 -- >>> t "0020/2"
--- "0020/02/01"
+-- "0020-02-01"
 -- >>> t "1000"
--- "1000/01/01"
+-- "1000-01-01"
 -- >>> t "4/2"
--- "2008/04/02"
+-- "2008-04-02"
 -- >>> t "2"
--- "2008/11/02"
+-- "2008-11-02"
 -- >>> t "January"
--- "2008/01/01"
+-- "2008-01-01"
 -- >>> t "feb"
--- "2008/02/01"
+-- "2008-02-01"
 -- >>> t "today"
--- "2008/11/26"
+-- "2008-11-26"
 -- >>> t "yesterday"
--- "2008/11/25"
+-- "2008-11-25"
 -- >>> t "tomorrow"
--- "2008/11/27"
+-- "2008-11-27"
 -- >>> t "this day"
--- "2008/11/26"
+-- "2008-11-26"
 -- >>> t "last day"
--- "2008/11/25"
+-- "2008-11-25"
 -- >>> t "next day"
--- "2008/11/27"
+-- "2008-11-27"
 -- >>> t "this week"  -- last monday
--- "2008/11/24"
+-- "2008-11-24"
 -- >>> t "last week"  -- previous monday
--- "2008/11/17"
+-- "2008-11-17"
 -- >>> t "next week"  -- next monday
--- "2008/12/01"
+-- "2008-12-01"
 -- >>> t "this month"
--- "2008/11/01"
+-- "2008-11-01"
 -- >>> t "last month"
--- "2008/10/01"
+-- "2008-10-01"
 -- >>> t "next month"
--- "2008/12/01"
+-- "2008-12-01"
 -- >>> t "this quarter"
--- "2008/10/01"
+-- "2008-10-01"
 -- >>> t "last quarter"
--- "2008/07/01"
+-- "2008-07-01"
 -- >>> t "next quarter"
--- "2009/01/01"
+-- "2009-01-01"
 -- >>> t "this year"
--- "2008/01/01"
+-- "2008-01-01"
 -- >>> t "last year"
--- "2007/01/01"
+-- "2007-01-01"
 -- >>> t "next year"
--- "2009/01/01"
+-- "2009-01-01"
 --
 -- t "last wed"
--- "2008/11/19"
+-- "2008-11-19"
 -- t "next friday"
--- "2008/11/28"
+-- "2008-11-28"
 -- t "next january"
--- "2009/01/01"
+-- "2009-01-01"
 --
 fixSmartDate :: Day -> SmartDate -> Day
 fixSmartDate refdate = fix
@@ -531,19 +543,19 @@ startofyear day = fromGregorian y 1 1 where (y,_,_) = toGregorian day
 -- Examples: lets take 2017-11-22. Year-long intervals covering it that
 -- starts before Nov 22 will start in 2017. However
 -- intervals that start after Nov 23rd should start in 2016:
--- >>> let wed22nd = parsedate "2017-11-22"          
+-- >>> let wed22nd = parsedate "2017-11-22"
 -- >>> nthdayofyearcontaining 11 21 wed22nd
--- 2017-11-21          
+-- 2017-11-21
 -- >>> nthdayofyearcontaining 11 22 wed22nd
--- 2017-11-22          
+-- 2017-11-22
 -- >>> nthdayofyearcontaining 11 23 wed22nd
--- 2016-11-23          
+-- 2016-11-23
 -- >>> nthdayofyearcontaining 12 02 wed22nd
--- 2016-12-02          
+-- 2016-12-02
 -- >>> nthdayofyearcontaining 12 31 wed22nd
--- 2016-12-31          
+-- 2016-12-31
 -- >>> nthdayofyearcontaining 1 1 wed22nd
--- 2017-01-01          
+-- 2017-01-01
 nthdayofyearcontaining :: Month -> MonthDay -> Day -> Day
 nthdayofyearcontaining m md date
   | not (validMonth $ show m)  = error' $ "nthdayofyearcontaining: invalid month "++show m
@@ -555,23 +567,23 @@ nthdayofyearcontaining m md date
         s = startofyear date
 
 -- | For given date d find month-long interval that starts on nth day of month
--- and covers it. 
+-- and covers it.
 -- The given day of month should be basically valid (1-31), or an error is raised.
 --
 -- Examples: lets take 2017-11-22. Month-long intervals covering it that
 -- start on 1st-22nd of month will start in Nov. However
 -- intervals that start on 23rd-30th of month should start in Oct:
--- >>> let wed22nd = parsedate "2017-11-22"          
+-- >>> let wed22nd = parsedate "2017-11-22"
 -- >>> nthdayofmonthcontaining 1 wed22nd
--- 2017-11-01          
+-- 2017-11-01
 -- >>> nthdayofmonthcontaining 12 wed22nd
--- 2017-11-12          
+-- 2017-11-12
 -- >>> nthdayofmonthcontaining 22 wed22nd
--- 2017-11-22          
+-- 2017-11-22
 -- >>> nthdayofmonthcontaining 23 wed22nd
--- 2017-10-23          
+-- 2017-10-23
 -- >>> nthdayofmonthcontaining 30 wed22nd
--- 2017-10-30          
+-- 2017-10-30
 nthdayofmonthcontaining :: MonthDay -> Day -> Day
 nthdayofmonthcontaining md date
   | not (validDay $ show md) = error' $ "nthdayofmonthcontaining: invalid day "  ++show md
@@ -582,22 +594,22 @@ nthdayofmonthcontaining md date
         s = startofmonth date
 
 -- | For given date d find week-long interval that starts on nth day of week
--- and covers it. 
+-- and covers it.
 --
 -- Examples: 2017-11-22 is Wed. Week-long intervals that cover it and
 -- start on Mon, Tue or Wed will start in the same week. However
--- intervals that start on Thu or Fri should start in prev week:          
--- >>> let wed22nd = parsedate "2017-11-22"          
+-- intervals that start on Thu or Fri should start in prev week:
+-- >>> let wed22nd = parsedate "2017-11-22"
 -- >>> nthdayofweekcontaining 1 wed22nd
--- 2017-11-20          
+-- 2017-11-20
 -- >>> nthdayofweekcontaining 2 wed22nd
 -- 2017-11-21
 -- >>> nthdayofweekcontaining 3 wed22nd
--- 2017-11-22          
+-- 2017-11-22
 -- >>> nthdayofweekcontaining 4 wed22nd
--- 2017-11-16          
+-- 2017-11-16
 -- >>> nthdayofweekcontaining 5 wed22nd
--- 2017-11-17          
+-- 2017-11-17
 nthdayofweekcontaining :: WeekDay -> Day -> Day
 nthdayofweekcontaining n d | nthOfSameWeek <= d = nthOfSameWeek
                            | otherwise = nthOfPrevWeek
@@ -606,12 +618,12 @@ nthdayofweekcontaining n d | nthOfSameWeek <= d = nthOfSameWeek
           s = startofweek d
 
 -- | For given date d find month-long interval that starts on nth weekday of month
--- and covers it. 
+-- and covers it.
 --
 -- Examples: 2017-11-22 is 3rd Wed of Nov. Month-long intervals that cover it and
 -- start on 1st-4th Wed will start in Nov. However
--- intervals that start on 4th Thu or Fri or later should start in Oct:          
--- >>> let wed22nd = parsedate "2017-11-22"          
+-- intervals that start on 4th Thu or Fri or later should start in Oct:
+-- >>> let wed22nd = parsedate "2017-11-22"
 -- >>> nthweekdayofmonthcontaining 1 3 wed22nd
 -- 2017-11-01
 -- >>> nthweekdayofmonthcontaining 3 2 wed22nd
@@ -630,12 +642,12 @@ nthweekdayofmonthcontaining n wd d | nthWeekdaySameMonth <= d  = nthWeekdaySameM
 
 -- | Advance to nth weekday wd after given start day s
 advancetonthweekday :: Int -> WeekDay -> Day -> Day
-advancetonthweekday n wd s = 
+advancetonthweekday n wd s =
   maybe err (addWeeks (n-1)) $ firstMatch (>=s) $ iterate (addWeeks 1) $ firstweekday s
   where
     err = error' "advancetonthweekday: should not happen"
     addWeeks k = addDays (7 * fromIntegral k)
-    firstMatch p = headMay . dropWhile (not . p) 
+    firstMatch p = headMay . dropWhile (not . p)
     firstweekday = addDays (fromIntegral wd-1) . startofweek
 
 ----------------------------------------------------------------------
@@ -643,7 +655,7 @@ advancetonthweekday n wd s =
 
 -- -- | Parse a couple of date-time string formats to a time type.
 -- parsedatetimeM :: String -> Maybe LocalTime
--- parsedatetimeM s = firstJust [
+-- parsedatetimeM s = asum [
 --     parseTime defaultTimeLocale "%Y/%m/%d %H:%M:%S" s,
 --     parseTime defaultTimeLocale "%Y-%m-%d %H:%M:%S" s
 --     ]
@@ -657,11 +669,14 @@ parsetime =
 #endif
 
 
--- | Parse a couple of date string formats to a time type.
+-- | Try to parse a couple of date string formats:
+-- `YYYY-MM-DD`, `YYYY/MM/DD` or `YYYY.MM.DD`, with leading zeros required.
+-- For internal use, not quite the same as the journal's "simple dates".
 parsedateM :: String -> Maybe Day
-parsedateM s = firstJust [
+parsedateM s = asum [
+     parsetime defaultTimeLocale "%Y-%m-%d" s,
      parsetime defaultTimeLocale "%Y/%m/%d" s,
-     parsetime defaultTimeLocale "%Y-%m-%d" s
+     parsetime defaultTimeLocale "%Y.%m.%d" s
      ]
 
 
@@ -670,7 +685,7 @@ parsedateM s = firstJust [
 -- parsedatetime s = fromMaybe (error' $ "could not parse timestamp \"" ++ s ++ "\"")
 --                             (parsedatetimeM s)
 
--- | Parse a YYYY-MM-DD or YYYY/MM/DD date string to a Day, or raise an error. For testing/debugging.
+-- | Like parsedateM, raising an error on parse failure.
 --
 -- >>> parsedate "2008/02/03"
 -- 2008-02-03
@@ -764,17 +779,17 @@ datesepchar :: TextParser m Char
 datesepchar = satisfy isDateSepChar
 
 isDateSepChar :: Char -> Bool
-isDateSepChar c = c == '/' || c == '-' || c == '.'
+isDateSepChar c = c == '-' || c == '/' || c == '.'
 
 validYear, validMonth, validDay :: String -> Bool
 validYear s = length s >= 4 && isJust (readMay s :: Maybe Year)
 validMonth s = maybe False (\n -> n>=1 && n<=12) $ readMay s
 validDay s = maybe False (\n -> n>=1 && n<=31) $ readMay s
 
-failIfInvalidYear, failIfInvalidMonth, failIfInvalidDay :: (Monad m) => String -> m ()
-failIfInvalidYear s  = unless (validYear s)  $ fail $ "bad year number: " ++ s
-failIfInvalidMonth s = unless (validMonth s) $ fail $ "bad month number: " ++ s
-failIfInvalidDay s   = unless (validDay s)   $ fail $ "bad day number: " ++ s
+failIfInvalidYear, failIfInvalidMonth, failIfInvalidDay :: (Fail.MonadFail m) => String -> m ()
+failIfInvalidYear s  = unless (validYear s)  $ Fail.fail $ "bad year number: " ++ s
+failIfInvalidMonth s = unless (validMonth s) $ Fail.fail $ "bad month number: " ++ s
+failIfInvalidDay s   = unless (validDay s)   $ Fail.fail $ "bad day number: " ++ s
 
 yyyymmdd :: TextParser m SmartDate
 yyyymmdd = do
@@ -834,7 +849,7 @@ md = do
   failIfInvalidDay d
   return ("",m,d)
 
--- These are compared case insensitively, and should all be kept lower case. 
+-- These are compared case insensitively, and should all be kept lower case.
 months         = ["january","february","march","april","may","june",
                   "july","august","september","october","november","december"]
 monthabbrevs   = ["jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"]
@@ -864,8 +879,8 @@ weekday = do
   wday <- T.toLower <$> (choice . map string' $ weekdays ++ weekdayabbrevs)
   case catMaybes $ [wday `elemIndex` weekdays, wday `elemIndex` weekdayabbrevs] of
     (i:_) -> return (i+1)
-    []    -> fail  $ "weekday: should not happen: attempted to find " <> 
-                      show wday <> " in " <> show (weekdays ++ weekdayabbrevs) 
+    []    -> Fail.fail $ "weekday: should not happen: attempted to find " <>
+                         show wday <> " in " <> show (weekdays ++ weekdayabbrevs)
 
 today,yesterday,tomorrow :: TextParser m SmartDate
 today     = string' "today"     >> return ("","","today")
@@ -892,50 +907,56 @@ lastthisnextthing = do
 
   return ("", T.unpack r, T.unpack p)
 
--- |
--- >>> let p = parsePeriodExpr (parsedate "2008/11/26")
+-- | Parse a period expression, specifying a date span and optionally
+-- a reporting interval. Requires a reference "today" date for
+-- resolving any relative start/end dates (only; it is not needed for
+-- parsing the reporting interval).
+--
+-- >>> let p = parsePeriodExpr (parsedate "2008-11-26")
 -- >>> p "from Aug to Oct"
--- Right (NoInterval,DateSpan 2008/08/01-2008/09/30)
+-- Right (NoInterval,DateSpan 2008-08-01..2008-09-30)
 -- >>> p "aug to oct"
--- Right (NoInterval,DateSpan 2008/08/01-2008/09/30)
+-- Right (NoInterval,DateSpan 2008-08-01..2008-09-30)
 -- >>> p "every 3 days in Aug"
--- Right (Days 3,DateSpan 2008/08)
+-- Right (Days 3,DateSpan 2008-08)
 -- >>> p "daily from aug"
--- Right (Days 1,DateSpan 2008/08/01-)
+-- Right (Days 1,DateSpan 2008-08-01..)
 -- >>> p "every week to 2009"
--- Right (Weeks 1,DateSpan -2008/12/31)
+-- Right (Weeks 1,DateSpan ..2008-12-31)
 -- >>> p "every 2nd day of month"
--- Right (DayOfMonth 2,DateSpan -)
+-- Right (DayOfMonth 2,DateSpan ..)
 -- >>> p "every 2nd day"
--- Right (DayOfMonth 2,DateSpan -)
+-- Right (DayOfMonth 2,DateSpan ..)
+-- >>> p "every 2nd day 2009.."
+-- Right (DayOfMonth 2,DateSpan 2009-01-01..)
 -- >>> p "every 2nd day 2009-"
--- Right (DayOfMonth 2,DateSpan 2009/01/01-)  
+-- Right (DayOfMonth 2,DateSpan 2009-01-01..)
 -- >>> p "every 29th Nov"
--- Right (DayOfYear 11 29,DateSpan -)
--- >>> p "every 29th nov -2009"
--- Right (DayOfYear 11 29,DateSpan -2008/12/31)
+-- Right (DayOfYear 11 29,DateSpan ..)
+-- >>> p "every 29th nov ..2009"
+-- Right (DayOfYear 11 29,DateSpan ..2008-12-31)
 -- >>> p "every nov 29th"
--- Right (DayOfYear 11 29,DateSpan -)
--- >>> p "every Nov 29th 2009-"
--- Right (DayOfYear 11 29,DateSpan 2009/01/01-)
+-- Right (DayOfYear 11 29,DateSpan ..)
+-- >>> p "every Nov 29th 2009.."
+-- Right (DayOfYear 11 29,DateSpan 2009-01-01..)
 -- >>> p "every 11/29 from 2009"
--- Right (DayOfYear 11 29,DateSpan 2009/01/01-)
+-- Right (DayOfYear 11 29,DateSpan 2009-01-01..)
 -- >>> p "every 2nd Thursday of month to 2009"
--- Right (WeekdayOfMonth 2 4,DateSpan -2008/12/31)
+-- Right (WeekdayOfMonth 2 4,DateSpan ..2008-12-31)
 -- >>> p "every 1st monday of month to 2009"
--- Right (WeekdayOfMonth 1 1,DateSpan -2008/12/31)
+-- Right (WeekdayOfMonth 1 1,DateSpan ..2008-12-31)
 -- >>> p "every tue"
--- Right (DayOfWeek 2,DateSpan -)
+-- Right (DayOfWeek 2,DateSpan ..)
 -- >>> p "every 2nd day of week"
--- Right (DayOfWeek 2,DateSpan -)
+-- Right (DayOfWeek 2,DateSpan ..)
 -- >>> p "every 2nd day of month"
--- Right (DayOfMonth 2,DateSpan -)
+-- Right (DayOfMonth 2,DateSpan ..)
 -- >>> p "every 2nd day"
--- Right (DayOfMonth 2,DateSpan -)
--- >>> p "every 2nd day 2009-"
--- Right (DayOfMonth 2,DateSpan 2009/01/01-)
--- >>> p "every 2nd day of month 2009-"
--- Right (DayOfMonth 2,DateSpan 2009/01/01-)
+-- Right (DayOfMonth 2,DateSpan ..)
+-- >>> p "every 2nd day 2009.."
+-- Right (DayOfMonth 2,DateSpan 2009-01-01..)
+-- >>> p "every 2nd day of month 2009.."
+-- Right (DayOfMonth 2,DateSpan 2009-01-01..)
 periodexprp :: Day -> TextParser m (Interval, DateSpan)
 periodexprp rdate = do
   skipMany spacenonewline
@@ -944,6 +965,7 @@ periodexprp rdate = do
                     (,) NoInterval <$> periodexprdatespanp rdate
                    ]
 
+-- Parse a reporting interval and a date span.
 intervalanddateperiodexprp :: Day -> TextParser m (Interval, DateSpan)
 intervalanddateperiodexprp rdate = do
   i <- reportingintervalp
@@ -1007,9 +1029,9 @@ reportingintervalp = choice' [
         string' "of"
         skipMany spacenonewline
         string' period
-        
+
       optOf_ period = optional $ try $ of_ period
-      
+
       nth = do n <- some digitChar
                choice' $ map string' ["st","nd","rd","th"]
                return $ read n
@@ -1046,13 +1068,13 @@ periodexprdatespanp rdate = choice $ map try [
 
 -- |
 -- -- >>> parsewith (doubledatespan (parsedate "2018/01/01") <* eof) "20180101-201804"
--- Right DateSpan 2018/01/01-2018/04/01
+-- Right DateSpan 2018-01-01..2018-04-01
 doubledatespanp :: Day -> TextParser m DateSpan
 doubledatespanp rdate = do
   optional (string' "from" >> skipMany spacenonewline)
   b <- smartdate
   skipMany spacenonewline
-  optional (choice [string' "to", string' "-"] >> skipMany spacenonewline)
+  optional (choice [string' "to", string "..", string' "-"] >> skipMany spacenonewline)
   DateSpan (Just $ fixSmartDate rdate b) . Just . fixSmartDate rdate <$> smartdate
 
 fromdatespanp :: Day -> TextParser m DateSpan
@@ -1064,14 +1086,14 @@ fromdatespanp rdate = do
     ,
     do
       d <- smartdate
-      string' "-"
+      choice [string "..", string' "-"]
       return d
     ]
   return $ DateSpan (Just $ fixSmartDate rdate b) Nothing
 
 todatespanp :: Day -> TextParser m DateSpan
 todatespanp rdate = do
-  choice [string' "to", string' "-"] >> skipMany spacenonewline
+  choice [string' "to", string' "until", string "..", string' "-"] >> skipMany spacenonewline
   DateSpan Nothing . Just . fixSmartDate rdate <$> smartdate
 
 justdatespanp :: Day -> TextParser m DateSpan
