@@ -31,11 +31,12 @@ import qualified Data.Text as T
 import Safe (headDef)
 import Text.Printf
 
-import Hledger.Utils.Test 
+import Hledger.Utils.Test
 import Hledger.Data.Types
 import Hledger.Data.Account
+import Hledger.Data.Dates (daysSpan)
 import Hledger.Data.Journal
-import Hledger.Data.Posting
+import Hledger.Data.Posting (postingDate)
 import Hledger.Query
 
 
@@ -53,10 +54,11 @@ nullledger = Ledger {
   laccounts = []
   }
 
--- | Filter a journal's transactions with the given query, then derive
--- a ledger containing the chart of accounts and balances. If the
--- query includes a depth limit, that will affect the ledger's
--- journal but not the ledger's account tree.
+-- | Filter a journal's transactions with the given query, then build
+-- a "Ledger", containing the journal plus the tree of all its
+-- accounts with their subaccount-inclusive and subaccount-exclusive
+-- balances. If the query includes a depth limit, the ledger's journal
+-- will be depth limited, but the ledger's account tree will not.
 ledgerFromJournal :: Query -> Journal -> Ledger
 ledgerFromJournal q j = nullledger{ljournal=j'', laccounts=as}
   where
@@ -99,7 +101,7 @@ ledgerPostings = journalPostings . ljournal
 -- | The (fully specified) date span containing all the ledger's (filtered) transactions,
 -- or DateSpan Nothing Nothing if there are none.
 ledgerDateSpan :: Ledger -> DateSpan
-ledgerDateSpan = postingsDateSpan . ledgerPostings
+ledgerDateSpan = daysSpan . map postingDate . ledgerPostings
 
 -- | All commodities used in this ledger.
 ledgerCommodities :: Ledger -> [CommoditySymbol]
@@ -108,12 +110,9 @@ ledgerCommodities = M.keys . jinferredcommodities . ljournal
 -- tests
 
 tests_Ledger =
-  tests
-    "Ledger"
-    [ tests
-        "ledgerFromJournal"
-        [ length (ledgerPostings $ ledgerFromJournal Any nulljournal) `is` 0
-        , length (ledgerPostings $ ledgerFromJournal Any samplejournal) `is` 13
-        , length (ledgerPostings $ ledgerFromJournal (Depth 2) samplejournal) `is` 7
-        ]
-    ]
+  tests "Ledger" [
+    test "ledgerFromJournal" $ do
+        length (ledgerPostings $ ledgerFromJournal Any nulljournal) @?= 0
+        length (ledgerPostings $ ledgerFromJournal Any samplejournal) @?= 13
+        length (ledgerPostings $ ledgerFromJournal (Depth 2) samplejournal) @?= 7
+  ]
