@@ -20,7 +20,6 @@ module Hledger.Data.Posting (
   vpost',
   nullsourcepos,
   nullassertion,
-  assertion,
   balassert,
   balassertTot,
   balassertParInc,
@@ -43,8 +42,6 @@ module Hledger.Data.Posting (
   postingDate2,
   isPostingInDateSpan,
   isPostingInDateSpan',
-  postingsDateSpan,
-  postingsDateSpan',
   -- * account name operations
   accountNamesFromPostings,
   accountNamePostingType,
@@ -54,20 +51,26 @@ module Hledger.Data.Posting (
   concatAccountNames,
   accountNameApplyAliases,
   accountNameApplyAliasesMemo,
-  -- * transaction description operations
-  transactionPayee,
-  transactionNote,
-  payeeAndNoteFromDescription,
+  -- * comment/tag operations
+  commentJoin,
+  commentAddTag,
+  commentAddTagNextLine,
   -- * arithmetic
   sumPostings,
   -- * rendering
   showPosting,
   -- * misc.
   showComment,
+  postingTransformAmount,
+  postingApplyValuation,
+  postingToCost,
   tests_Posting
 )
 where
-import Data.List
+
+import Data.Foldable (asum)
+import Data.List.Extra (nubSort)
+import qualified Data.Map as M
 import Data.Maybe
 import Data.MemoUgly (memo)
 #if !(MIN_VERSION_base(4,11,0))
@@ -78,11 +81,12 @@ import qualified Data.Text as T
 import Data.Time.Calendar
 import Safe
 
-import Hledger.Utils 
+import Hledger.Utils
 import Hledger.Data.Types
 import Hledger.Data.Amount
 import Hledger.Data.AccountName
 import Hledger.Data.Dates (nulldate, spanContainsDate)
+import Hledger.Data.Valuation
 
 
 
@@ -123,14 +127,13 @@ vpost' acc amt ass = (post' acc amt ass){ptype=VirtualPosting, pbalanceassertion
 nullsourcepos :: GenericSourcePos
 nullsourcepos = JournalSourcePos "" (1,1)
 
-nullassertion, assertion :: BalanceAssertion
+nullassertion :: BalanceAssertion
 nullassertion = BalanceAssertion
                   {baamount=nullamt
                   ,batotal=False
                   ,bainclusive=False
                   ,baposition=nullsourcepos
                   }
-assertion = nullassertion
 
 -- | Make a partial, exclusive balance assertion.
 balassert :: Amount -> Maybe BalanceAssertion
@@ -187,7 +190,7 @@ hasBalanceAssignment p = not (hasAmount p) && isJust (pbalanceassertion p)
 
 -- | Sorted unique account names referenced by these postings.
 accountNamesFromPostings :: [Posting] -> [AccountName]
-accountNamesFromPostings = nub . sort . map paccount
+accountNamesFromPostings = nubSort . map paccount
 
 sumPostings :: [Posting] -> MixedAmount
 sumPostings = sumStrict . map pamount
@@ -195,32 +198,31 @@ sumPostings = sumStrict . map pamount
 -- | Remove all prices of a posting
 removePrices :: Posting -> Posting
 removePrices p = p{ pamount = Mixed $ remove <$> amounts (pamount p) }
-  where remove a = a { aprice = NoPrice }
+  where remove a = a { aprice = Nothing }
 
 -- | Get a posting's (primary) date - it's own primary date if specified,
 -- otherwise the parent transaction's primary date, or the null date if
 -- there is no parent transaction.
 postingDate :: Posting -> Day
-postingDate p = fromMaybe txndate $ pdate p
-    where
-      txndate = maybe nulldate tdate $ ptransaction p
+postingDate p = fromMaybe nulldate $ asum dates
+    where dates = [ pdate p, tdate <$> ptransaction p ]
 
 -- | Get a posting's secondary (secondary) date, which is the first of:
 -- posting's secondary date, transaction's secondary date, posting's
 -- primary date, transaction's primary date, or the null date if there is
 -- no parent transaction.
 postingDate2 :: Posting -> Day
-postingDate2 p = headDef nulldate $ catMaybes dates
-  where dates = [pdate2 p
-                ,maybe Nothing tdate2 $ ptransaction p
-                ,pdate p
-                ,fmap tdate (ptransaction p)
+postingDate2 p = fromMaybe nulldate $ asum dates
+  where dates = [ pdate2 p
+                , tdate2 =<< ptransaction p
+                , pdate p
+                , tdate <$> ptransaction p
                 ]
 
 -- | Get a posting's status. This is cleared or pending if those are
 -- explicitly set on the posting, otherwise the status of its parent
 -- transaction, or unmarked if there is no parent transaction. (Note
--- the ambiguity, unmarked can mean "posting and transaction are both 
+-- the ambiguity, unmarked can mean "posting and transaction are both
 -- unmarked" or "posting is unmarked and don't know about the transaction".
 postingStatus :: Posting -> Status
 postingStatus Posting{pstatus=s, ptransaction=mt}
@@ -228,23 +230,6 @@ postingStatus Posting{pstatus=s, ptransaction=mt}
                                Nothing -> Unmarked
   | otherwise = s
 
-transactionPayee :: Transaction -> Text
-transactionPayee = fst . payeeAndNoteFromDescription . tdescription
-
-transactionNote :: Transaction -> Text
-transactionNote = snd . payeeAndNoteFromDescription . tdescription
-
--- | Parse a transaction's description into payee and note (aka narration) fields,
--- assuming a convention of separating these with | (like Beancount).
--- Ie, everything up to the first | is the payee, everything after it is the note.
--- When there's no |, payee == note == description.
-payeeAndNoteFromDescription :: Text -> (Text,Text)
-payeeAndNoteFromDescription t
-  | T.null n = (t, t)
-  | otherwise = (textstrip p, textstrip $ T.drop 1 n)
-  where
-    (p, n) = T.span (/= '|') t
-
 -- | Tags for this posting including any inherited from its parent transaction.
 postingAllTags :: Posting -> [Tag]
 postingAllTags p = ptags p ++ maybe [] ttags (ptransaction p)
@@ -260,7 +245,7 @@ relatedPostings _ = []
 
 -- | Does this posting fall within the given date span ?
 isPostingInDateSpan :: DateSpan -> Posting -> Bool
-isPostingInDateSpan s = spanContainsDate s . postingDate
+isPostingInDateSpan = isPostingInDateSpan' PrimaryDate
 
 -- --date2-sensitive version, separate for now to avoid disturbing multiBalanceReport.
 isPostingInDateSpan' :: WhichDate -> DateSpan -> Posting -> Bool
@@ -268,22 +253,7 @@ isPostingInDateSpan' PrimaryDate   s = spanContainsDate s . postingDate
 isPostingInDateSpan' SecondaryDate s = spanContainsDate s . postingDate2
 
 isEmptyPosting :: Posting -> Bool
-isEmptyPosting = isZeroMixedAmount . pamount
-
--- | Get the minimal date span which contains all the postings, or the
--- null date span if there are none.
-postingsDateSpan :: [Posting] -> DateSpan
-postingsDateSpan [] = DateSpan Nothing Nothing
-postingsDateSpan ps = DateSpan (Just $ postingDate $ head ps') (Just $ addDays 1 $ postingDate $ last ps')
-    where ps' = sortOn postingDate ps
-
--- --date2-sensitive version, as above.
-postingsDateSpan' :: WhichDate -> [Posting] -> DateSpan
-postingsDateSpan' _  [] = DateSpan Nothing Nothing
-postingsDateSpan' wd ps = DateSpan (Just $ postingdate $ head ps') (Just $ addDays 1 $ postingdate $ last ps')
-    where
-      ps' = sortOn postingdate ps
-      postingdate = if wd == PrimaryDate then postingDate else postingDate2
+isEmptyPosting = mixedAmountLooksZero . pamount
 
 -- AccountName stuff that depends on PostingType
 
@@ -331,6 +301,7 @@ accountNameApplyAliases aliases a = accountNameWithPostingType atype aname'
 -- | Memoising version of accountNameApplyAliases, maybe overkill.
 accountNameApplyAliasesMemo :: [AccountAlias] -> AccountName -> AccountName
 accountNameApplyAliasesMemo aliases = memo (accountNameApplyAliases aliases)
+  -- XXX re-test this memoisation
 
 -- aliasMatches :: AccountAlias -> AccountName -> Bool
 -- aliasMatches (BasicAlias old _) a = old `isAccountNamePrefixOf` a
@@ -342,36 +313,96 @@ aliasReplace (BasicAlias old new) a
   | otherwise = a
 aliasReplace (RegexAlias re repl) a = T.pack $ regexReplaceCIMemo re repl $ T.unpack a -- XXX
 
+-- | Apply a specified valuation to this posting's amount, using the
+-- provided price oracle, commodity styles, reference dates, and
+-- whether this is for a multiperiod report or not. See
+-- amountApplyValuation.
+postingApplyValuation :: PriceOracle -> M.Map CommoditySymbol AmountStyle -> Day -> Maybe Day -> Day -> Bool -> Posting -> ValuationType -> Posting
+postingApplyValuation priceoracle styles periodlast mreportlast today ismultiperiod p v =
+  case v of
+    AtCost    Nothing            -> postingToCost styles p
+    AtCost    mc                 -> postingValueAtDate priceoracle styles mc periodlast $ postingToCost styles p
+    AtThen    mc                 -> postingValueAtDate priceoracle styles mc (postingDate p) p
+    AtEnd     mc                 -> postingValueAtDate priceoracle styles mc periodlast p
+    AtNow     mc                 -> postingValueAtDate priceoracle styles mc today p
+    AtDefault mc | ismultiperiod -> postingValueAtDate priceoracle styles mc periodlast p
+    AtDefault mc                 -> postingValueAtDate priceoracle styles mc (fromMaybe today mreportlast) p
+    AtDate d  mc                 -> postingValueAtDate priceoracle styles mc d p
+
+-- | Convert this posting's amount to cost, and apply the appropriate amount styles.
+postingToCost :: M.Map CommoditySymbol AmountStyle -> Posting -> Posting
+postingToCost styles p@Posting{pamount=a} = p{pamount=styleMixedAmount styles $ mixedAmountCost a}
+
+-- | Convert this posting's amount to market value in the given commodity,
+-- or the default valuation commodity, at the given valuation date,
+-- using the given market price oracle.
+-- When market prices available on that date are not sufficient to
+-- calculate the value, amounts are left unchanged.
+postingValueAtDate :: PriceOracle -> M.Map CommoditySymbol AmountStyle -> Maybe CommoditySymbol -> Day -> Posting -> Posting
+postingValueAtDate priceoracle styles mc d p = postingTransformAmount (mixedAmountValueAtDate priceoracle styles mc d) p
+
+-- | Apply a transform function to this posting's amount.
+postingTransformAmount :: (MixedAmount -> MixedAmount) -> Posting -> Posting
+postingTransformAmount f p@Posting{pamount=a} = p{pamount=f a}
+
+-- | Join two parts of a comment, eg a tag and another tag, or a tag
+-- and a non-tag, on a single line. Interpolates a comma and space
+-- unless one of the parts is empty.
+commentJoin :: Text -> Text -> Text
+commentJoin c1 c2
+  | T.null c1 = c2
+  | T.null c2 = c1
+  | otherwise = c1 <> ", " <> c2
+
+-- | Add a tag to a comment, comma-separated from any prior content.
+-- A space is inserted following the colon, before the value.
+commentAddTag :: Text -> Tag -> Text
+commentAddTag c (t,v)
+  | T.null c' = tag
+  | otherwise = c' `commentJoin` tag
+  where
+    c'  = textchomp c
+    tag = t <> ": " <> v
+
+-- | Add a tag on its own line to a comment, preserving any prior content.
+-- A space is inserted following the colon, before the value.
+commentAddTagNextLine :: Text -> Tag -> Text
+commentAddTagNextLine cmt (t,v) =
+  cmt <> if "\n" `T.isSuffixOf` cmt then "" else "\n" <> t <> ": " <> v 
+
 
 -- tests
 
 tests_Posting = tests "Posting" [
 
-  tests "accountNamePostingType" [
-    accountNamePostingType "a" `is` RegularPosting
-    ,accountNamePostingType "(a)" `is` VirtualPosting
-    ,accountNamePostingType "[a]" `is` BalancedVirtualPosting
-  ]
+  test "accountNamePostingType" $ do
+    accountNamePostingType "a" @?= RegularPosting
+    accountNamePostingType "(a)" @?= VirtualPosting
+    accountNamePostingType "[a]" @?= BalancedVirtualPosting
 
- ,tests "accountNameWithoutPostingType" [
-    accountNameWithoutPostingType "(a)" `is` "a"
-  ]
+ ,test "accountNameWithoutPostingType" $ do
+    accountNameWithoutPostingType "(a)" @?= "a"
 
- ,tests "accountNameWithPostingType" [
-    accountNameWithPostingType VirtualPosting "[a]" `is` "(a)"
-  ]
+ ,test "accountNameWithPostingType" $ do
+    accountNameWithPostingType VirtualPosting "[a]" @?= "(a)"
 
- ,tests "joinAccountNames" [
-    "a" `joinAccountNames` "b:c" `is` "a:b:c"
-    ,"a" `joinAccountNames` "(b:c)" `is` "(a:b:c)"
-    ,"[a]" `joinAccountNames` "(b:c)" `is` "[a:b:c]"
-    ,"" `joinAccountNames` "a" `is` "a"
-  ]
+ ,test "joinAccountNames" $ do
+    "a" `joinAccountNames` "b:c" @?= "a:b:c"
+    "a" `joinAccountNames` "(b:c)" @?= "(a:b:c)"
+    "[a]" `joinAccountNames` "(b:c)" @?= "[a:b:c]"
+    "" `joinAccountNames` "a" @?= "a"
 
- ,tests "concatAccountNames" [
-    concatAccountNames [] `is` ""
-    ,concatAccountNames ["a","(b)","[c:d]"] `is` "(a:b:c:d)"
-  ]
+ ,test "concatAccountNames" $ do
+    concatAccountNames [] @?= ""
+    concatAccountNames ["a","(b)","[c:d]"] @?= "(a:b:c:d)"
+
+ ,test "commentAddTag" $ do
+    commentAddTag "" ("a","") @?= "a: "
+    commentAddTag "[1/2]" ("a","") @?= "[1/2], a: "
+
+ ,test "commentAddTagNextLine" $ do
+    commentAddTagNextLine "" ("a","") @?= "\na: "
+    commentAddTagNextLine "[1/2]" ("a","") @?= "[1/2]\na: "
 
  ]
 
