@@ -29,7 +29,7 @@ import Data.Default
 import Data.Functor (($>))
 import Data.List (intercalate)
 import Text.Blaze (ToMarkup(..))
---XXX https://hackage.haskell.org/package/containers/docs/Data-Map.html 
+--XXX https://hackage.haskell.org/package/containers/docs/Data-Map.html
 --Note: You should use Data.Map.Strict instead of this module if:
 --You will eventually need all the values stored.
 --The stored values don't represent large virtual data structures to be lazily computed.
@@ -154,12 +154,13 @@ instance ToMarkup Quantity
  where
    toMarkup = toMarkup . show
 
--- | An amount's price (none, per unit, or total) in another commodity.
--- The price amount should always be positive.
-data Price = NoPrice | UnitPrice Amount | TotalPrice Amount 
+-- | An amount's per-unit or total cost/selling price in another
+-- commodity, as recorded in the journal entry eg with @ or @@.
+-- Docs call this "transaction price". The amount is always positive.
+data AmountPrice = UnitPrice Amount | TotalPrice Amount
   deriving (Eq,Ord,Typeable,Data,Generic,Show)
 
-instance NFData Price
+instance NFData AmountPrice
 
 -- | Display style for an amount.
 data AmountStyle = AmountStyle {
@@ -207,7 +208,7 @@ data Amount = Amount {
       aismultiplier :: Bool,            -- ^ kludge: a flag marking this amount and posting as a multiplier
                                         --   in a TMPostingRule. In a regular Posting, should always be false.
       astyle      :: AmountStyle,
-      aprice      :: Price            -- ^ the (fixed, transaction-specific) price for this amount, if any
+      aprice      :: Maybe AmountPrice  -- ^ the (fixed, transaction-specific) price for this amount, if any
     } deriving (Eq,Ord,Typeable,Data,Generic,Show)
 
 instance NFData Amount
@@ -299,7 +300,7 @@ data Posting = Posting {
                                                     --   Tying this knot gets tedious, Maybe makes it easier/optional.
       poriginal         :: Maybe Posting            -- ^ When this posting has been transformed in some way
                                                     --   (eg its amount or price was inferred, or the account name was
-                                                    --   changed by a pivot or budget report), this references the original 
+                                                    --   changed by a pivot or budget report), this references the original
                                                     --   untransformed posting (which will have Nothing in this field).
     } deriving (Typeable,Data,Generic)
 
@@ -356,10 +357,10 @@ data Transaction = Transaction {
 instance NFData Transaction
 
 -- | A transaction modifier rule. This has a query which matches postings
--- in the journal, and a list of transformations to apply to those 
+-- in the journal, and a list of transformations to apply to those
 -- postings or their transactions. Currently there is one kind of transformation:
--- the TMPostingRule, which adds a posting ("auto posting") to the transaction, 
--- optionally setting its amount to the matched posting's amount multiplied by a constant. 
+-- the TMPostingRule, which adds a posting ("auto posting") to the transaction,
+-- optionally setting its amount to the matched posting's amount multiplied by a constant.
 data TransactionModifier = TransactionModifier {
       tmquerytxt :: Text,
       tmpostingrules :: [TMPostingRule]
@@ -381,8 +382,8 @@ type TMPostingRule = Posting
 -- | A periodic transaction rule, describing a transaction that recurs.
 data PeriodicTransaction = PeriodicTransaction {
       ptperiodexpr   :: Text,     -- ^ the period expression as written
-      ptinterval     :: Interval, -- ^ the interval at which this transaction recurs 
-      ptspan         :: DateSpan, -- ^ the (possibly unbounded) period during which this transaction recurs. Contains a whole number of intervals. 
+      ptinterval     :: Interval, -- ^ the interval at which this transaction recurs
+      ptspan         :: DateSpan, -- ^ the (possibly unbounded) period during which this transaction recurs. Contains a whole number of intervals.
       --
       ptstatus       :: Status,   -- ^ some of Transaction's fields
       ptcode         :: Text,
@@ -420,14 +421,32 @@ data TimeclockEntry = TimeclockEntry {
 
 instance NFData TimeclockEntry
 
+-- | A market price declaration made by the journal format's P directive.
+-- It declares two things: a historical exchange rate between two commodities,
+-- and an amount display style for the second commodity.
+data PriceDirective = PriceDirective {
+   pddate      :: Day
+  ,pdcommodity :: CommoditySymbol
+  ,pdamount    :: Amount
+  } deriving (Eq,Ord,Typeable,Data,Generic,Show)
+        -- Show instance derived in Amount.hs (XXX why ?)
+
+instance NFData PriceDirective
+
+-- | A historical market price (exchange rate) from one commodity to another.
+-- A more concise form of a PriceDirective, without the amount display info.
 data MarketPrice = MarketPrice {
-      mpdate      :: Day,
-      mpcommodity :: CommoditySymbol,
-      mpamount    :: Amount
-    } deriving (Eq,Ord,Typeable,Data,Generic) -- , Show in Amount.hs
+   mpdate :: Day                -- ^ Date on which this price becomes effective.
+  ,mpfrom :: CommoditySymbol    -- ^ The commodity being converted from.
+  ,mpto   :: CommoditySymbol    -- ^ The commodity being converted to.
+  ,mprate :: Quantity           -- ^ One unit of the "from" commodity is worth this quantity of the "to" commodity.
+  } deriving (Eq,Ord,Typeable,Data,Generic)
+        -- Show instance derived in Amount.hs (XXX why ?)
 
 instance NFData MarketPrice
 
+-- additional valuation-related types in Valuation.hs
+
 -- | A Journal, containing transactions and various other things.
 -- The basic data model for hledger.
 --
@@ -448,11 +467,12 @@ data Journal = Journal {
   ,jparsetimeclockentries :: [TimeclockEntry]                       -- ^ timeclock sessions which have not been clocked out
   ,jincludefilestack      :: [FilePath]
   -- principal data
-  ,jdeclaredaccounts      :: [(AccountName,AccountDeclarationInfo)] -- ^ Accounts declared by account directives, in parse order (after journal finalisation) 
-  ,jdeclaredaccounttypes  :: M.Map AccountType [AccountName]        -- ^ Accounts whose type has been declared in account directives (usually 5 top-level accounts) 
+  ,jdeclaredaccounts      :: [(AccountName,AccountDeclarationInfo)] -- ^ Accounts declared by account directives, in parse order (after journal finalisation)
+  ,jdeclaredaccounttypes  :: M.Map AccountType [AccountName]        -- ^ Accounts whose type has been declared in account directives (usually 5 top-level accounts)
   ,jcommodities           :: M.Map CommoditySymbol Commodity        -- ^ commodities and formats declared by commodity directives
-  ,jinferredcommodities   :: M.Map CommoditySymbol AmountStyle      -- ^ commodities and formats inferred from journal amounts  TODO misnamed - jusedstyles
-  ,jmarketprices          :: [MarketPrice]
+  ,jinferredcommodities   :: M.Map CommoditySymbol AmountStyle      -- ^ commodities and formats inferred from journal amounts  TODO misnamed, should be eg jusedstyles
+  ,jpricedirectives       :: [PriceDirective]                       -- ^ Declarations of market prices by P directives, in parse order (after journal finalisation)
+  ,jinferredmarketprices  :: [MarketPrice]                          -- ^ Market prices implied by transactions, in parse order (after journal finalisation)
   ,jtxnmodifiers          :: [TransactionModifier]
   ,jperiodictxns          :: [PeriodicTransaction]
   ,jtxns                  :: [Transaction]
@@ -509,12 +529,12 @@ data Account = Account {
   ,aibalance                 :: MixedAmount    -- ^ this account's balance, including subaccounts
   } deriving (Typeable, Data, Generic)
 
--- | Whether an account's balance is normally a positive number (in 
--- accounting terms, a debit balance) or a negative number (credit balance). 
+-- | Whether an account's balance is normally a positive number (in
+-- accounting terms, a debit balance) or a negative number (credit balance).
 -- Assets and expenses are normally positive (debit), while liabilities, equity
 -- and income are normally negative (credit).
 -- https://en.wikipedia.org/wiki/Normal_balance
-data NormalSign = NormallyPositive | NormallyNegative deriving (Show, Data, Eq) 
+data NormalSign = NormallyPositive | NormallyNegative deriving (Show, Data, Eq)
 
 -- | A Ledger has the journal it derives from, and the accounts
 -- derived from that. Accounts are accessible both list-wise and
