@@ -1,10 +1,6 @@
---- * doc
--- Lines beginning "--- *" are collapsible orgstruct nodes. Emacs users,
--- (add-hook 'haskell-mode-hook
---   (lambda () (set-variable 'orgstruct-heading-prefix-regexp "--- " t))
---   'orgstruct-mode)
--- and press TAB on nodes to expand/collapse.
-
+--- * -*- outline-regexp:"--- *"; -*-
+--- ** doc
+-- In Emacs, use TAB on lines beginning with "-- *" to collapse/expand sections.
 {-|
 
 A reader for hledger's journal file format
@@ -21,19 +17,32 @@ reader should handle many ledger files as well. Example:
 
 Journal format supports the include directive which can read files in
 other formats, so the other file format readers need to be importable
-here.  Some low-level journal syntax parsers which those readers also
-use are therefore defined separately in Hledger.Read.Common, avoiding
-import cycles.
+and invocable here.
+
+Some important parts of journal parsing are therefore kept in
+Hledger.Read.Common, to avoid import cycles.
 
 -}
 
---- * module
+--- ** language
 
-{-# LANGUAGE CPP, RecordWildCards, NamedFieldPuns, NoMonoLocalBinds, ScopedTypeVariables, FlexibleContexts, TupleSections, OverloadedStrings, PackageImports #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE NamedFieldPuns #-}
+{-# LANGUAGE NoMonoLocalBinds #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE PackageImports #-}
+{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TupleSections #-}
 
+--- ** exports
 module Hledger.Read.JournalReader (
---- * exports
 
+  -- * Reader-finding utils
+  findReader,
+  splitReaderPrefix,
+  
   -- * Reader
   reader,
 
@@ -55,23 +64,34 @@ module Hledger.Read.JournalReader (
   postingp,
   statusp,
   emptyorcommentlinep,
-  followingcommentp
+  followingcommentp,
+  accountaliasp
 
   -- * Tests
   ,tests_JournalReader
 )
 where
---- * imports
-import Prelude ()
-import "base-compat-batteries" Prelude.Compat hiding (readFile)
+
+--- ** imports
+-- import qualified Prelude (fail)
+-- import "base-compat-batteries" Prelude.Compat hiding (fail, readFile)
+import qualified "base-compat-batteries" Control.Monad.Fail.Compat as Fail (fail)
 import qualified Control.Exception as C
-import Control.Monad
+import Control.Monad (forM_, when, void)
+import Control.Monad.IO.Class (MonadIO, liftIO)
 import Control.Monad.Except (ExceptT(..), runExceptT)
-import Control.Monad.State.Strict
+import Control.Monad.State.Strict (get,modify',put)
+import Control.Monad.Trans.Class (lift)
+import Data.Char (toLower)
+import Data.Either (isRight)
 import qualified Data.Map.Strict as M
+#if !(MIN_VERSION_base(4,11,0))
+import Data.Semigroup ((<>))
+#endif
 import Data.Text (Text)
 import Data.String
 import Data.List
+import Data.Maybe
 import qualified Data.Text as T
 import Data.Time.Calendar
 import Data.Time.LocalTime
@@ -85,21 +105,68 @@ import "Glob" System.FilePath.Glob hiding (match)
 
 import Hledger.Data
 import Hledger.Read.Common
-import Hledger.Read.TimeclockReader (timeclockfilep)
-import Hledger.Read.TimedotReader (timedotfilep)
 import Hledger.Utils
 
+import qualified Hledger.Read.TimedotReader as TimedotReader (reader)
+import qualified Hledger.Read.TimeclockReader as TimeclockReader (reader)
+import qualified Hledger.Read.CsvReader as CsvReader (reader)
+
+--- ** doctest setup
 -- $setup
 -- >>> :set -XOverloadedStrings
 
---- * reader
+--- ** reader finding utilities
+-- Defined here rather than Hledger.Read so that we can use them in includedirectivep below.
 
-reader :: Reader
+-- The available journal readers, each one handling a particular data format.
+readers' :: MonadIO m => [Reader m]
+readers' = [
+  reader
+ ,TimeclockReader.reader
+ ,TimedotReader.reader
+ ,CsvReader.reader
+--  ,LedgerReader.reader
+ ]
+
+readerNames :: [String]
+readerNames = map rFormat (readers'::[Reader IO])
+
+-- | @findReader mformat mpath@
+--
+-- Find the reader named by @mformat@, if provided.
+-- Or, if a file path is provided, find the first reader that handles
+-- its file extension, if any.
+findReader :: MonadIO m => Maybe StorageFormat -> Maybe FilePath -> Maybe (Reader m)
+findReader Nothing Nothing     = Nothing
+findReader (Just fmt) _        = headMay [r | r <- readers', rFormat r == fmt]
+findReader Nothing (Just path) =
+  case prefix of
+    Just fmt -> headMay [r | r <- readers', rFormat r == fmt]
+    Nothing  -> headMay [r | r <- readers', ext `elem` rExtensions r]
+  where
+    (prefix,path') = splitReaderPrefix path
+    ext            = map toLower $ drop 1 $ takeExtension path'
+
+-- | A file path optionally prefixed by a reader name and colon
+-- (journal:, csv:, timedot:, etc.).
+type PrefixedFilePath = FilePath
+
+-- | If a filepath is prefixed by one of the reader names and a colon,
+-- split that off. Eg "csv:-" -> (Just "csv", "-").
+splitReaderPrefix :: PrefixedFilePath -> (Maybe String, FilePath)
+splitReaderPrefix f =
+  headDef (Nothing, f)
+  [(Just r, drop (length r + 1) f) | r <- readerNames, (r++":") `isPrefixOf` f]
+
+--- ** reader
+
+reader :: MonadIO m => Reader m
 reader = Reader
   {rFormat     = "journal"
   ,rExtensions = ["journal", "j", "hledger", "ledger"]
-  ,rParser     = parse
-  ,rExperimental = False
+  ,rReadFn     = parse
+  ,rParser    = journalp  -- no need to add command line aliases like journalp'
+                           -- when called as a subparser I think
   }
 
 -- | Parse and post-process a "Journal" from hledger's journal file
@@ -107,9 +174,9 @@ reader = Reader
 parse :: InputOpts -> FilePath -> Text -> ExceptT String IO Journal
 parse iopts = parseAndFinaliseJournal journalp' iopts
   where
-    journalp' = do 
+    journalp' = do
       -- reverse parsed aliases to ensure that they are applied in order given on commandline
-      mapM_ addAccountAlias (reverse $ aliasesFromOpts iopts) 
+      mapM_ addAccountAlias (reverse $ aliasesFromOpts iopts)
       journalp
 
 -- | Get the account name aliases from options, if any.
@@ -117,8 +184,8 @@ aliasesFromOpts :: InputOpts -> [AccountAlias]
 aliasesFromOpts = map (\a -> fromparse $ runParser accountaliasp ("--alias "++quoteIfNeeded a) $ T.pack a)
                   . aliases_
 
---- * parsers
---- ** journal
+--- ** parsers
+--- *** journal
 
 -- | A journal parser. Accumulates and returns a "ParsedJournal",
 -- which should be finalised/validated before use.
@@ -143,12 +210,12 @@ addJournalItemP =
     , transactionp          >>= modify' . addTransaction
     , transactionmodifierp  >>= modify' . addTransactionModifier
     , periodictransactionp  >>= modify' . addPeriodicTransaction
-    , marketpricedirectivep >>= modify' . addMarketPrice
+    , marketpricedirectivep >>= modify' . addPriceDirective
     , void (lift emptyorcommentlinep)
     , void (lift multilinecommentp)
     ] <?> "transaction or directive"
 
---- ** directives
+--- *** directives
 
 -- | Parse any journal directive and update the parse state accordingly.
 -- Cf http://hledger.org/manual.html#directives,
@@ -173,19 +240,23 @@ directivep = (do
    ]
   ) <?> "directive"
 
+-- | Parse an include directive. include's argument is an optionally
+-- file-format-prefixed file path or glob pattern. In the latter case,
+-- the prefix is applied to each matched path. Examples:
+-- foo.j, foo/bar.j, timedot:foo/2020*.md
 includedirectivep :: MonadIO m => ErroringJournalParser m ()
 includedirectivep = do
   string "include"
   lift (skipSome spacenonewline)
-  filename <- T.unpack <$> takeWhileP Nothing (/= '\n') -- don't consume newline yet
-
+  prefixedglob <- T.unpack <$> takeWhileP Nothing (/= '\n') -- don't consume newline yet
   parentoff <- getOffset
   parentpos <- getSourcePos
-
-  filepaths <- getFilePaths parentoff parentpos filename
-
-  forM_ filepaths $ parseChild parentpos
-
+  let (mprefix,glob) = splitReaderPrefix prefixedglob
+  paths <- getFilePaths parentoff parentpos glob
+  let prefixedpaths = case mprefix of
+        Nothing  -> paths
+        Just fmt -> map ((fmt++":")++) paths
+  forM_ prefixedpaths $ parseChild parentpos
   void newline
 
   where
@@ -208,23 +279,24 @@ includedirectivep = do
             else customFailure $ parseErrorAt parseroff $
                    "No existing files match pattern: " ++ filename
 
-    parseChild :: MonadIO m => SourcePos -> FilePath -> ErroringJournalParser m ()
-    parseChild parentpos filepath = do
-      parentj <- get
+    parseChild :: MonadIO m => SourcePos -> PrefixedFilePath -> ErroringJournalParser m ()
+    parseChild parentpos prefixedpath = do
+      let (_mprefix,filepath) = splitReaderPrefix prefixedpath
 
+      parentj <- get
       let parentfilestack = jincludefilestack parentj
       when (filepath `elem` parentfilestack) $
-        fail ("Cyclic include: " ++ filepath)
+        Fail.fail ("Cyclic include: " ++ filepath)
 
       childInput <- lift $ readFilePortably filepath
                             `orRethrowIOError` (show parentpos ++ " reading " ++ filepath)
       let initChildj = newJournalWithParseStateFrom filepath parentj
 
-      let parser = choiceInState
-            [ journalp
-            , timeclockfilep
-            , timedotfilep
-            ] -- can't include a csv file yet, that reader is special
+      -- Choose a reader/format based on the file path, or fall back
+      -- on journal. Duplicating readJournal a bit here.
+      let r = fromMaybe reader $ findReader Nothing (Just prefixedpath)
+          parser = rParser r
+      dbg7IO "trying reader" (rFormat r)
       updatedChildj <- journalAddFile (filepath, childInput) <$>
                         parseIncludeFile parser initChildj filepath childInput
 
@@ -232,7 +304,7 @@ includedirectivep = do
       put $ updatedChildj <> parentj
 
     newJournalWithParseStateFrom :: FilePath -> Journal -> Journal
-    newJournalWithParseStateFrom filepath j = mempty{
+    newJournalWithParseStateFrom filepath j = nulljournal{
       jparsedefaultyear      = jparsedefaultyear j
       ,jparsedefaultcommodity = jparsedefaultcommodity j
       ,jparseparentaccounts   = jparseparentaccounts j
@@ -250,7 +322,7 @@ orRethrowIOError io msg = do
   eResult <- liftIO $ (Right <$> io) `C.catch` \(e::C.IOException) -> pure $ Left $ printf "%s:\n%s" msg (show e)
   case eResult of
     Right res -> pure res
-    Left errMsg -> fail errMsg
+    Left errMsg -> Fail.fail errMsg
 
 -- Parse an account directive, adding its info to the journal's
 -- list of account declarations.
@@ -267,12 +339,12 @@ accountdirectivep = do
   -- maybe an account type code (ALERX) after two or more spaces
   -- XXX added in 1.11, deprecated in 1.13, remove in 1.14
   mtypecode :: Maybe Char <- lift $ optional $ try $ do
-    skipSome spacenonewline -- at least one more space in addition to the one consumed by modifiedaccountp 
+    skipSome spacenonewline -- at least one more space in addition to the one consumed by modifiedaccountp
     choice $ map char "ALERX"
 
   -- maybe a comment, on this and/or following lines
   (cmt, tags) <- lift transactioncommentp
-  
+
   -- maybe Ledger-style subdirectives (ignored)
   skipMany indentedlinep
 
@@ -353,13 +425,21 @@ commoditydirectiveonelinep = do
     pure $ (off, amount)
   lift (skipMany spacenonewline)
   _ <- lift followingcommentp
-  let comm = Commodity{csymbol=acommodity, cformat=Just $ dbg2 "style from commodity directive" astyle}
+  let comm = Commodity{csymbol=acommodity, cformat=Just $ dbg7 "style from commodity directive" astyle}
   if asdecimalpoint astyle == Nothing
   then customFailure $ parseErrorAt off pleaseincludedecimalpoint
   else modify' (\j -> j{jcommodities=M.insert acommodity comm $ jcommodities j})
 
 pleaseincludedecimalpoint :: String
-pleaseincludedecimalpoint = "to avoid ambiguity, please include a decimal separator in commodity directives"
+pleaseincludedecimalpoint = chomp $ unlines [
+   "Please include a decimal point or decimal comma in commodity directives,"
+  ,"to help us parse correctly. It may be followed by zero or more decimal digits."
+  ,"Examples:"
+  ,"commodity $1000.            ; no thousands mark, decimal period, no decimals"
+  ,"commodity 1.234,00 ARS      ; period at thousands, decimal comma, 2 decimals"
+  ,"commodity EUR 1 000,000     ; space at thousands, decimal comma, 3 decimals"
+  ,"commodity INR1,23,45,678.0  ; comma at thousands/lakhs/crores, decimal period, 1 decimal"
+  ]
 
 -- | Parse a multi-line commodity directive, containing 0 or more format subdirectives.
 --
@@ -386,10 +466,10 @@ formatdirectivep expectedsym = do
   Amount{acommodity,astyle} <- amountp
   _ <- lift followingcommentp
   if acommodity==expectedsym
-    then 
+    then
       if asdecimalpoint astyle == Nothing
       then customFailure $ parseErrorAt off pleaseincludedecimalpoint
-      else return $ dbg2 "style from format subdirective" astyle
+      else return $ dbg7 "style from format subdirective" astyle
     else customFailure $ parseErrorAt off $
          printf "commodity directive symbol \"%s\" and format directive symbol \"%s\" should be the same" expectedsym acommodity
 
@@ -486,7 +566,7 @@ defaultcommoditydirectivep = do
   then customFailure $ parseErrorAt off pleaseincludedecimalpoint
   else setDefaultCommodityAndStyle (acommodity, astyle)
 
-marketpricedirectivep :: JournalParser m MarketPrice
+marketpricedirectivep :: JournalParser m PriceDirective
 marketpricedirectivep = do
   char 'P' <?> "market price"
   lift (skipMany spacenonewline)
@@ -496,7 +576,7 @@ marketpricedirectivep = do
   lift (skipMany spacenonewline)
   price <- amountp
   lift restofline
-  return $ MarketPrice date symbol price
+  return $ PriceDirective date symbol price
 
 ignoredpricecommoditydirectivep :: JournalParser m ()
 ignoredpricecommoditydirectivep = do
@@ -518,8 +598,9 @@ commodityconversiondirectivep = do
   lift restofline
   return ()
 
---- ** transactions
+--- *** transactions
 
+-- | Parse a transaction modifier (auto postings) rule.
 transactionmodifierp :: JournalParser m TransactionModifier
 transactionmodifierp = do
   char '=' <?> "modifier transaction"
@@ -529,10 +610,10 @@ transactionmodifierp = do
   postings <- postingsp Nothing
   return $ TransactionModifier querytxt postings
 
--- | Parse a periodic transaction
+-- | Parse a periodic transaction rule.
 --
 -- This reuses periodexprp which parses period expressions on the command line.
--- This is awkward because periodexprp supports relative and partial dates, 
+-- This is awkward because periodexprp supports relative and partial dates,
 -- which we don't really need here, and it doesn't support the notion of a
 -- default year set by a Y directive, which we do need to consider here.
 -- We resolve it as follows: in periodic transactions' period expressions,
@@ -546,12 +627,12 @@ periodictransactionp = do
   lift $ skipMany spacenonewline
   -- a period expression
   off <- getOffset
-  
+
   -- if there's a default year in effect, use Y/1/1 as base for partial/relative dates
   today <- liftIO getCurrentDay
   mdefaultyear <- getYear
   let refdate = case mdefaultyear of
-                  Nothing -> today 
+                  Nothing -> today
                   Just y  -> fromGregorian y 1 1
   periodExcerpt <- lift $ excerpt_ $
                     singlespacedtextsatisfyingp (\c -> c /= ';' && c /= '\n')
@@ -576,7 +657,7 @@ periodictransactionp = do
   case checkPeriodicTransactionStartDate interval span periodtxt of
     Just e -> customFailure $ parseErrorAt off e
     Nothing -> pure ()
-  
+
   status <- lift statusp <?> "cleared status"
   code <- lift codep <?> "transaction code"
   description <- lift $ T.strip <$> descriptionp
@@ -614,7 +695,7 @@ transactionp = do
   let sourcepos = journalSourcePos startpos endpos
   return $ txnTieKnot $ Transaction 0 "" sourcepos date edate status code description comment tags postings
 
---- ** postings
+--- *** postings
 
 -- Parse the following whitespace-beginning lines as postings, posting
 -- tags, and/or comments (inferring year, if needed, from the given date).
@@ -641,8 +722,7 @@ postingp mTransactionYear = do
   lift (skipMany spacenonewline)
   amount <- option missingmixedamt $ Mixed . (:[]) <$> amountp
   lift (skipMany spacenonewline)
-  massertion <- optional $ balanceassertionp
-  _ <- fixedlotpricep
+  massertion <- optional balanceassertionp
   lift (skipMany spacenonewline)
   (comment,tags,mdate,mdate2) <- lift $ postingcommentp mTransactionYear
   return posting
@@ -657,53 +737,53 @@ postingp mTransactionYear = do
    , pbalanceassertion=massertion
    }
 
---- * tests
+--- ** tests
 
 tests_JournalReader = tests "JournalReader" [
 
    let p = lift accountnamep :: JournalParser IO AccountName in
    tests "accountnamep" [
-     test "basic" $ expectParse p "a:b:c"
-    ,_test "empty inner component" $ expectParseError p "a::c" ""  -- TODO
-    ,_test "empty leading component" $ expectParseError p ":b:c" "x"
-    ,_test "empty trailing component" $ expectParseError p "a:b:" "x"
+     test "basic" $ assertParse p "a:b:c"
+    -- ,test "empty inner component" $ assertParseError p "a::c" ""  -- TODO
+    -- ,test "empty leading component" $ assertParseError p ":b:c" "x"
+    -- ,test "empty trailing component" $ assertParseError p "a:b:" "x"
     ]
 
   -- "Parse a date in YYYY/MM/DD format.
   -- Hyphen (-) and period (.) are also allowed as separators.
   -- The year may be omitted if a default year has been set.
   -- Leading zeroes may be omitted."
-  ,test "datep" $ do
-    test "YYYY/MM/DD" $ expectParseEq datep "2018/01/01" (fromGregorian 2018 1 1)
-    test "YYYY-MM-DD" $ expectParse datep "2018-01-01"
-    test "YYYY.MM.DD" $ expectParse datep "2018.01.01"
-    test "yearless date with no default year" $ expectParseError datep "1/1" "current year is unknown"
-    test "yearless date with default year" $ do 
+  ,tests "datep" [
+     test "YYYY/MM/DD" $ assertParseEq datep "2018/01/01" (fromGregorian 2018 1 1)
+    ,test "YYYY-MM-DD" $ assertParse datep "2018-01-01"
+    ,test "YYYY.MM.DD" $ assertParse datep "2018.01.01"
+    ,test "yearless date with no default year" $ assertParseError datep "1/1" "current year is unknown"
+    ,test "yearless date with default year" $ do
       let s = "1/1"
-      ep <- parseWithState mempty{jparsedefaultyear=Just 2018} datep s
-      either (fail.("parse error at "++).customErrorBundlePretty) (const ok) ep
-    test "no leading zero" $ expectParse datep "2018/1/1"
-
+      ep <- parseWithState nulljournal{jparsedefaultyear=Just 2018} datep s
+      either (assertFailure . ("parse error at "++) . customErrorBundlePretty) (const $ return ()) ep
+    ,test "no leading zero" $ assertParse datep "2018/1/1"
+    ]
   ,test "datetimep" $ do
-      let
-        good = expectParse datetimep
-        bad = (\t -> expectParseError datetimep t "")
-      good "2011/1/1 00:00"
-      good "2011/1/1 23:59:59"
-      bad "2011/1/1"
-      bad "2011/1/1 24:00:00"
-      bad "2011/1/1 00:60:00"
-      bad "2011/1/1 00:00:60"
-      bad "2011/1/1 3:5:7"
-      test "timezone is parsed but ignored" $ do
-        let t = LocalTime (fromGregorian 2018 1 1) (TimeOfDay 0 0 (fromIntegral 0))
-        expectParseEq datetimep "2018/1/1 00:00-0800" t
-        expectParseEq datetimep "2018/1/1 00:00+1234" t
+     let
+       good = assertParse datetimep
+       bad  = (\t -> assertParseError datetimep t "")
+     good "2011/1/1 00:00"
+     good "2011/1/1 23:59:59"
+     bad "2011/1/1"
+     bad "2011/1/1 24:00:00"
+     bad "2011/1/1 00:60:00"
+     bad "2011/1/1 00:00:60"
+     bad "2011/1/1 3:5:7"
+     -- timezone is parsed but ignored
+     let t = LocalTime (fromGregorian 2018 1 1) (TimeOfDay 0 0 (fromIntegral 0))
+     assertParseEq datetimep "2018/1/1 00:00-0800" t
+     assertParseEq datetimep "2018/1/1 00:00+1234" t
 
   ,tests "periodictransactionp" [
 
-    test "more period text in comment after one space" $ expectParseEq periodictransactionp
-      "~ monthly from 2018/6 ;In 2019 we will change this\n" 
+    test "more period text in comment after one space" $ assertParseEq periodictransactionp
+      "~ monthly from 2018/6 ;In 2019 we will change this\n"
       nullperiodictransaction {
          ptperiodexpr  = "monthly from 2018/6"
         ,ptinterval    = Months 1
@@ -712,8 +792,8 @@ tests_JournalReader = tests "JournalReader" [
         ,ptcomment     = "In 2019 we will change this\n"
         }
 
-    ,test "more period text in description after two spaces" $ expectParseEq periodictransactionp
-      "~ monthly from 2018/6   In 2019 we will change this\n" 
+    ,test "more period text in description after two spaces" $ assertParseEq periodictransactionp
+      "~ monthly from 2018/6   In 2019 we will change this\n"
       nullperiodictransaction {
          ptperiodexpr  = "monthly from 2018/6"
         ,ptinterval    = Months 1
@@ -722,7 +802,7 @@ tests_JournalReader = tests "JournalReader" [
         ,ptcomment     = ""
         }
 
-    ,test "Next year in description" $ expectParseEq periodictransactionp
+    ,test "Next year in description" $ assertParseEq periodictransactionp
       "~ monthly  Next year blah blah\n"
       nullperiodictransaction {
          ptperiodexpr  = "monthly"
@@ -732,7 +812,7 @@ tests_JournalReader = tests "JournalReader" [
         ,ptcomment     = ""
         }
 
-    ,test "Just date, no description" $ expectParseEq periodictransactionp
+    ,test "Just date, no description" $ assertParseEq periodictransactionp
       "~ 2019-01-04\n"
       nullperiodictransaction {
          ptperiodexpr  = "2019-01-04"
@@ -742,22 +822,22 @@ tests_JournalReader = tests "JournalReader" [
         ,ptcomment     = ""
         }
 
-    ,test "Just date, no description + empty transaction comment" $ expectParse periodictransactionp
+    ,test "Just date, no description + empty transaction comment" $ assertParse periodictransactionp
       "~ 2019-01-04\n  ;\n  a  1\n  b\n"
 
     ]
 
   ,tests "postingp" [
-     test "basic" $ expectParseEq (postingp Nothing) 
+     test "basic" $ assertParseEq (postingp Nothing)
       "  expenses:food:dining  $10.00   ; a: a a \n   ; b: b b \n"
       posting{
-        paccount="expenses:food:dining", 
-        pamount=Mixed [usd 10], 
-        pcomment="a: a a\nb: b b\n", 
+        paccount="expenses:food:dining",
+        pamount=Mixed [usd 10],
+        pcomment="a: a a\nb: b b\n",
         ptags=[("a","a a"), ("b","b b")]
         }
 
-    ,test "posting dates" $ expectParseEq (postingp Nothing) 
+    ,test "posting dates" $ assertParseEq (postingp Nothing)
       " a  1. ; date:2012/11/28, date2=2012/11/29,b:b\n"
       nullposting{
          paccount="a"
@@ -768,27 +848,36 @@ tests_JournalReader = tests "JournalReader" [
         ,pdate2=Nothing  -- Just $ fromGregorian 2012 11 29
         }
 
-    ,test "posting dates bracket syntax" $ expectParseEq (postingp Nothing) 
+    ,test "posting dates bracket syntax" $ assertParseEq (postingp Nothing)
       " a  1. ; [2012/11/28=2012/11/29]\n"
       nullposting{
          paccount="a"
         ,pamount=Mixed [num 1]
         ,pcomment="[2012/11/28=2012/11/29]\n"
         ,ptags=[]
-        ,pdate= Just $ fromGregorian 2012 11 28 
+        ,pdate= Just $ fromGregorian 2012 11 28
         ,pdate2=Just $ fromGregorian 2012 11 29
         }
 
-    ,test "quoted commodity symbol with digits" $ expectParse (postingp Nothing) "  a  1 \"DE123\"\n"
+    ,test "quoted commodity symbol with digits" $ assertParse (postingp Nothing) "  a  1 \"DE123\"\n"
 
-    ,test "balance assertion and fixed lot price" $ expectParse (postingp Nothing) "  a  1 \"DE123\" =$1 { =2.2 EUR} \n"
+    ,test "only lot price" $ assertParse (postingp Nothing) "  a  1A {1B}\n"
+    ,test "fixed lot price" $ assertParse (postingp Nothing) "  a  1A {=1B}\n"
+    ,test "total lot price" $ assertParse (postingp Nothing) "  a  1A {{1B}}\n"
+    ,test "fixed total lot price, and spaces" $ assertParse (postingp Nothing) "  a  1A {{  =  1B }}\n"
+    ,test "lot price before transaction price" $ assertParse (postingp Nothing) "  a  1A {1B} @ 1B\n"
+    ,test "lot price after transaction price" $ assertParse (postingp Nothing) "  a  1A @ 1B {1B}\n"
+    ,test "lot price after balance assertion not allowed" $ assertParseError (postingp Nothing) "  a  1A @ 1B = 1A {1B}\n" "unexpected '{'"
+    ,test "only lot date" $ assertParse (postingp Nothing) "  a  1A [2000-01-01]\n"
+    ,test "transaction price, lot price, lot date" $ assertParse (postingp Nothing) "  a  1A @ 1B {1B} [2000-01-01]\n"
+    ,test "lot date, lot price, transaction price" $ assertParse (postingp Nothing) "  a  1A [2000-01-01] {1B} @ 1B\n"
 
-    ,test "balance assertion over entire contents of account" $ expectParse (postingp Nothing) "  a  $1 == $1\n"
+    ,test "balance assertion over entire contents of account" $ assertParse (postingp Nothing) "  a  $1 == $1\n"
     ]
 
   ,tests "transactionmodifierp" [
 
-    test "basic" $ expectParseEq transactionmodifierp 
+    test "basic" $ assertParseEq transactionmodifierp
       "= (some value expr)\n some:postings  1.\n"
       nulltransactionmodifier {
         tmquerytxt = "(some value expr)"
@@ -797,10 +886,10 @@ tests_JournalReader = tests "JournalReader" [
     ]
 
   ,tests "transactionp" [
-  
-     test "just a date" $ expectParseEq transactionp "2015/1/1\n" nulltransaction{tdate=fromGregorian 2015 1 1}
-  
-    ,test "more complex" $ expectParseEq transactionp 
+
+     test "just a date" $ assertParseEq transactionp "2015/1/1\n" nulltransaction{tdate=fromGregorian 2015 1 1}
+
+    ,test "more complex" $ assertParseEq transactionp
       (T.unlines [
         "2012/05/14=2012/05/15 (code) desc  ; tcomment1",
         "    ; tcomment2",
@@ -833,28 +922,28 @@ tests_JournalReader = tests "JournalReader" [
             }
           ]
       }
-  
+
     ,test "parses a well-formed transaction" $
-      expect $ isRight $ rjp transactionp $ T.unlines
+      assertBool "" $ isRight $ rjp transactionp $ T.unlines
         ["2007/01/28 coopportunity"
         ,"    expenses:food:groceries                   $47.18"
         ,"    assets:checking                          $-47.18"
         ,""
         ]
-  
+
     ,test "does not parse a following comment as part of the description" $
-      expectParseEqOn transactionp "2009/1/1 a ;comment\n b 1\n" tdescription "a"
-  
-    ,test "transactionp parses a following whitespace line" $
-      expect $ isRight $ rjp transactionp $ T.unlines
+      assertParseEqOn transactionp "2009/1/1 a ;comment\n b 1\n" tdescription "a"
+
+    ,test "parses a following whitespace line" $
+      assertBool "" $ isRight $ rjp transactionp $ T.unlines
         ["2012/1/1"
         ,"  a  1"
         ,"  b"
         ," "
         ]
 
-    ,test "transactionp parses an empty transaction comment following whitespace line" $
-      expect $ isRight $ rjp transactionp $ T.unlines
+    ,test "parses an empty transaction comment following whitespace line" $
+      assertBool "" $ isRight $ rjp transactionp $ T.unlines
         ["2012/1/1"
         ,"  ;"
         ,"  a  1"
@@ -863,7 +952,7 @@ tests_JournalReader = tests "JournalReader" [
         ]
 
     ,test "comments everywhere, two postings parsed" $
-      expectParseEqOn transactionp 
+      assertParseEqOn transactionp
         (T.unlines
           ["2009/1/1 x  ; transaction comment"
           ," a  1  ; posting 1 comment"
@@ -873,74 +962,74 @@ tests_JournalReader = tests "JournalReader" [
           ])
         (length . tpostings)
         2
-  
+
     ]
 
   -- directives
 
   ,tests "directivep" [
-    test "supports !" $ do 
-      expectParseE directivep "!account a\n"
-      expectParseE directivep "!D 1.0\n"
-    ]
+    test "supports !" $ do
+        assertParseE directivep "!account a\n"
+        assertParseE directivep "!D 1.0\n"
+     ]
 
-  ,test "accountdirectivep" $ do
-    test "with-comment"       $ expectParse accountdirectivep "account a:b  ; a comment\n"
-    test "does-not-support-!" $ expectParseError accountdirectivep "!account a:b\n" ""
-    test "account-type-code"  $ expectParse accountdirectivep "account a:b  A\n"
-    test "account-type-tag"   $ expectParseStateOn accountdirectivep "account a:b  ; type:asset\n"
-      jdeclaredaccounts
-      [("a:b", AccountDeclarationInfo{adicomment          = "type:asset\n"
-                                     ,aditags             = [("type","asset")]
-                                     ,adideclarationorder = 1
-                                     })
+  ,tests "accountdirectivep" [
+       test "with-comment"       $ assertParse accountdirectivep "account a:b  ; a comment\n"
+      ,test "does-not-support-!" $ assertParseError accountdirectivep "!account a:b\n" ""
+      ,test "account-type-code"  $ assertParse accountdirectivep "account a:b  A\n"
+      ,test "account-type-tag"   $ assertParseStateOn accountdirectivep "account a:b  ; type:asset\n"
+        jdeclaredaccounts
+        [("a:b", AccountDeclarationInfo{adicomment          = "type:asset\n"
+                                       ,aditags             = [("type","asset")]
+                                       ,adideclarationorder = 1
+                                       })
+        ]
       ]
 
   ,test "commodityconversiondirectivep" $ do
-     expectParse commodityconversiondirectivep "C 1h = $50.00\n"
+     assertParse commodityconversiondirectivep "C 1h = $50.00\n"
 
   ,test "defaultcommoditydirectivep" $ do
-     expectParse defaultcommoditydirectivep "D $1,000.0\n"
-     expectParseError defaultcommoditydirectivep "D $1000\n" "please include a decimal separator"
+      assertParse defaultcommoditydirectivep "D $1,000.0\n"
+      assertParseError defaultcommoditydirectivep "D $1000\n" "Please include a decimal point or decimal comma"
 
-  ,test "defaultyeardirectivep" $ do
-    test "1000" $ expectParse defaultyeardirectivep "Y 1000" -- XXX no \n like the others
-    test "999" $ expectParseError defaultyeardirectivep "Y 999" "bad year number"
-    test "12345" $ expectParse defaultyeardirectivep "Y 12345"
+  ,tests "defaultyeardirectivep" [
+      test "1000" $ assertParse defaultyeardirectivep "Y 1000" -- XXX no \n like the others
+     ,test "999" $ assertParseError defaultyeardirectivep "Y 999" "bad year number"
+     ,test "12345" $ assertParse defaultyeardirectivep "Y 12345"
+     ]
 
   ,test "ignoredpricecommoditydirectivep" $ do
-     expectParse ignoredpricecommoditydirectivep "N $\n"
+     assertParse ignoredpricecommoditydirectivep "N $\n"
 
-  ,test "includedirectivep" $ do
-    test "include" $ expectParseErrorE includedirectivep "include nosuchfile\n" "No existing files match pattern: nosuchfile"
-    test "glob" $ expectParseErrorE includedirectivep "include nosuchfile*\n" "No existing files match pattern: nosuchfile*"
+  ,tests "includedirectivep" [
+      test "include" $ assertParseErrorE includedirectivep "include nosuchfile\n" "No existing files match pattern: nosuchfile"
+     ,test "glob" $ assertParseErrorE includedirectivep "include nosuchfile*\n" "No existing files match pattern: nosuchfile*"
+     ]
 
-  ,test "marketpricedirectivep" $ expectParseEq marketpricedirectivep
+  ,test "marketpricedirectivep" $ assertParseEq marketpricedirectivep
     "P 2017/01/30 BTC $922.83\n"
-    MarketPrice{
-      mpdate      = fromGregorian 2017 1 30,
-      mpcommodity = "BTC",
-      mpamount    = usd 922.83
+    PriceDirective{
+      pddate      = fromGregorian 2017 1 30,
+      pdcommodity = "BTC",
+      pdamount    = usd 922.83
       }
 
   ,test "tagdirectivep" $ do
-     expectParse tagdirectivep "tag foo \n"
+     assertParse tagdirectivep "tag foo \n"
 
   ,test "endtagdirectivep" $ do
-     expectParse endtagdirectivep "end tag \n"
-     expectParse endtagdirectivep "pop \n"
-
+      assertParse endtagdirectivep "end tag \n"
+      assertParse endtagdirectivep "pop \n"
 
   ,tests "journalp" [
-    test "empty file" $ expectParseEqE journalp "" nulljournal
+    test "empty file" $ assertParseEqE journalp "" nulljournal
     ]
 
    -- these are defined here rather than in Common so they can use journalp
-  ,tests "parseAndFinaliseJournal" [
-    test "basic" $ do
-        ej <- io $ runExceptT $ parseAndFinaliseJournal journalp definputopts "" "2019-1-1\n"
-        let Right j = ej
-        expectEqPP [""] $ journalFilePaths j
-   ]
+  ,test "parseAndFinaliseJournal" $ do
+      ej <- runExceptT $ parseAndFinaliseJournal journalp definputopts "" "2019-1-1\n"
+      let Right j = ej
+      assertEqual "" [""] $ journalFilePaths j
 
   ]
