@@ -4,21 +4,16 @@ Balance report, used by the balance command.
 
 -}
 
-
-
-
-
-
-
-
-{-# LANGUAGE FlexibleInstances, ScopedTypeVariables, OverloadedStrings #-}
+{-# LANGUAGE FlexibleInstances, RecordWildCards, ScopedTypeVariables, OverloadedStrings #-}
 
 module Hledger.Reports.BalanceReport (
   BalanceReport,
   BalanceReportItem,
   balanceReport,
   flatShowsExclusiveBalance,
-  sortAccountItemsLike, 
+  sortAccountItemsLike,
+  unifyMixedAmount,
+  perdivide,
 
   -- * Tests
   tests_BalanceReport
@@ -33,16 +28,10 @@ import Data.Time.Calendar
 import Hledger.Data
 import Hledger.Read (mamountp')
 import Hledger.Query
-import Hledger.Utils 
+import Hledger.Utils
 import Hledger.Reports.ReportOptions
 
 
-
-
-
-
-
-
 -- | A simple balance report. It has:
 --
 -- 1. a list of items, one per account, each containing:
@@ -77,70 +66,101 @@ flatShowsExclusiveBalance    = True
 -- This is like PeriodChangeReport with a single column (but more mature,
 -- eg this can do hierarchical display).
 balanceReport :: ReportOpts -> Query -> Journal -> BalanceReport
-balanceReport opts q j = 
-  (if invert_ opts then brNegate else id) $ 
-  (sorteditems, total)
+balanceReport ropts@ReportOpts{..} q j =
+  (if invert_ then brNegate  else id) $
+  (mappedsorteditems, mappedtotal)
     where
-      -- dbg1 = const id -- exclude from debug output
-      dbg1 s = let p = "balanceReport" in Hledger.Utils.dbg1 (p++" "++s)  -- add prefix in debug output
+      -- dbg = const id -- exclude from debug output
+      dbg s = let p = "balanceReport" in Hledger.Utils.dbg4 (p++" "++s)  -- add prefix in debug output
+      dbg' s = let p = "balanceReport" in Hledger.Utils.dbg5 (p++" "++s)  -- add prefix in debug output
 
-      accts = ledgerRootAccount $ ledgerFromJournal q $ journalSelectingAmountFromOpts opts j
-      accts' :: [Account]
+      -- Get all the summed accounts & balances, according to the query, as an account tree.
+      -- If doing cost valuation, amounts will be converted to cost first.
+      accttree = ledgerRootAccount $ ledgerFromJournal q $ journalSelectingAmountFromOpts ropts j
+
+      -- For other kinds of valuation, convert the summed amounts to value,
+      -- per hledger_options.m4.md "Effect of --value on reports".
+      valuedaccttree = mapAccounts avalue accttree
+        where
+          avalue a@Account{..} = a{aebalance=maybevalue aebalance, aibalance=maybevalue aibalance}
+            where
+              maybevalue = maybe id applyvaluation value_
+                where
+                  applyvaluation = mixedAmountApplyValuation priceoracle styles periodlast mreportlast today multiperiod
+                    where
+                      priceoracle = journalPriceOracle infer_value_ j
+                      styles = journalCommodityStyles j
+                      periodlast = fromMaybe
+                                   (error' "balanceReport: expected a non-empty journal") $ -- XXX shouldn't happen
+                                   reportPeriodOrJournalLastDay ropts j
+                      mreportlast = reportPeriodLastDay ropts
+                      today = fromMaybe (error' "balanceReport: could not pick a valuation date, ReportOpts today_ is unset") today_
+                      multiperiod = interval_ /= NoInterval
+
+      -- Modify this tree for display - depth limit, boring parents, zeroes - and convert to a list.
+      displayaccts :: [Account]
           | queryDepth q == 0 =
-                         dbg1 "accts" $
-                         take 1 $ clipAccountsAndAggregate (queryDepth q) $ flattenAccounts accts
-          | flat_ opts = dbg1 "accts" $
+                         dbg' "displayaccts" $
+                         take 1 $ clipAccountsAndAggregate (queryDepth q) $ flattenAccounts valuedaccttree
+          | flat_ ropts = dbg' "displayaccts" $
                          filterzeros $
                          filterempty $
-                         drop 1 $ clipAccountsAndAggregate (queryDepth q) $ flattenAccounts accts
-          | otherwise  = dbg1 "accts" $
+                         drop 1 $ clipAccountsAndAggregate (queryDepth q) $ flattenAccounts valuedaccttree
+          | otherwise  = dbg' "displayaccts" $
                          filter (not.aboring) $
                          drop 1 $ flattenAccounts $
                          markboring $
                          prunezeros $
-                         sortAccountTreeByAmount (fromMaybe NormallyPositive $ normalbalance_ opts) $
-                         clipAccounts (queryDepth q) accts
+                         sortAccountTreeByAmount (fromMaybe NormallyPositive normalbalance_) $
+                         clipAccounts (queryDepth q) valuedaccttree
           where
-            balance   = if flat_ opts then aebalance else aibalance
-            filterzeros = if empty_ opts then id else filter (not . isZeroMixedAmount . balance)
-            filterempty = filter (\a -> anumpostings a > 0 || not (isZeroMixedAmount (balance a)))
-            prunezeros  = if empty_ opts then id else fromMaybe nullacct . pruneAccounts (isZeroMixedAmount . balance)
-            markboring  = if no_elide_ opts then id else markBoringParentAccounts
+            balance     = if flat_ ropts then aebalance else aibalance
+            filterzeros = if empty_ then id else filter (not . mixedAmountLooksZero . balance)
+            filterempty = filter (\a -> anumpostings a > 0 || not (mixedAmountLooksZero (balance a)))
+            prunezeros  = if empty_ then id else fromMaybe nullacct . pruneAccounts (mixedAmountLooksZero . balance)
+            markboring  = if no_elide_ then id else markBoringParentAccounts
 
-      items = dbg1 "items" $ map (balanceReportItem opts q) accts'
+      -- Make a report row for each account.
+      items = dbg "items" $ map (balanceReportItem ropts q) displayaccts
 
-      -- now sort items like MultiBalanceReport, except 
-      -- sorting a tree by amount was more easily done above
-      sorteditems 
-        | sort_amount_ opts && tree_ opts = items
-        | sort_amount_ opts               = sortFlatBRByAmount items
-        | otherwise                       = sortBRByAccountDeclaration items
-      
-        where    
-          -- Sort the report rows, representing a flat account list, by row total. 
+      -- Sort report rows (except sorting by amount in tree mode, which was done above).
+      sorteditems
+        | sort_amount_ && tree_ ropts = items
+        | sort_amount_                = sortFlatBRByAmount items
+        | otherwise                   = sortBRByAccountDeclaration items
+        where
+          -- Sort the report rows, representing a flat account list, by row total.
           sortFlatBRByAmount :: [BalanceReportItem] -> [BalanceReportItem]
           sortFlatBRByAmount = sortBy (maybeflip $ comparing (normaliseMixedAmountSquashPricesForDisplay . fourth4))
             where
-              maybeflip = if normalbalance_ opts == Just NormallyNegative then id else flip
-    
-          -- Sort the report rows by account declaration order then account name. 
+              maybeflip = if normalbalance_ == Just NormallyNegative then id else flip
+          -- Sort the report rows by account declaration order then account name.
           sortBRByAccountDeclaration :: [BalanceReportItem] -> [BalanceReportItem]
           sortBRByAccountDeclaration rows = sortedrows
-            where 
+            where
               anamesandrows = [(first4 r, r) | r <- rows]
               anames = map fst anamesandrows
-              sortedanames = sortAccountNamesByDeclaration j (tree_ opts) anames
-              sortedrows = sortAccountItemsLike sortedanames anamesandrows 
+              sortedanames = sortAccountNamesByDeclaration j (tree_ ropts) anames
+              sortedrows = sortAccountItemsLike sortedanames anamesandrows
 
-      total | not (flat_ opts) = dbg1 "total" $ sum [amt | (_,_,indent,amt) <- items, indent == 0]
-            | otherwise        = dbg1 "total" $
-                                 if flatShowsExclusiveBalance
-                                 then sum $ map fourth4 items
-                                 else sum $ map aebalance $ clipAccountsAndAggregate 1 accts'
+      -- Calculate the grand total.
+      total | not (flat_ ropts) = dbg "total" $ sum [amt | (_,_,indent,amt) <- items, indent == 0]
+            | otherwise         = dbg "total" $
+                                  if flatShowsExclusiveBalance
+                                  then sum $ map fourth4 items
+                                  else sum $ map aebalance $ clipAccountsAndAggregate 1 displayaccts
+      
+      -- Calculate percentages if needed.
+      mappedtotal | percent_  = dbg "mappedtotal" $ total `perdivide` total
+                  | otherwise = total
+      mappedsorteditems | percent_ =
+                            dbg "mappedsorteditems" $
+                            map (\(fname, sname, indent, amount) -> (fname, sname, indent, amount `perdivide` total)) sorteditems
+                        | otherwise = sorteditems
 
 -- | A sorting helper: sort a list of things (eg report rows) keyed by account name
 -- to match the provided ordering of those same account names.
-sortAccountItemsLike :: [AccountName] -> [(AccountName, b)] -> [b] 
+sortAccountItemsLike :: [AccountName] -> [(AccountName, b)] -> [b]
 sortAccountItemsLike sortedas items =
   concatMap (\a -> maybe [] (:[]) $ lookup a items) sortedas
 
@@ -150,7 +170,7 @@ sortAccountItemsLike sortedas items =
 markBoringParentAccounts :: Account -> Account
 markBoringParentAccounts = tieAccountParents . mapAccounts mark
   where
-    mark a | length (asubs a) == 1 && isZeroMixedAmount (aebalance a) = a{aboring=True}
+    mark a | length (asubs a) == 1 && mixedAmountLooksZero (aebalance a) = a{aboring=True}
            | otherwise = a
 
 balanceReportItem :: ReportOpts -> Query -> Account -> BalanceReportItem
@@ -176,10 +196,39 @@ balanceReportItem opts q a
 
 -- | Flip the sign of all amounts in a BalanceReport.
 brNegate :: BalanceReport -> BalanceReport
-brNegate (is, tot) = (map brItemNegate is, -tot) 
+brNegate (is, tot) = (map brItemNegate is, -tot)
   where
     brItemNegate (a, a', d, amt) = (a, a', d, -amt)
 
+-- | Helper to unify a MixedAmount to a single commodity value.
+-- Like normaliseMixedAmount, this consolidates amounts of the same commodity
+-- and discards zero amounts; but this one insists on simplifying to
+-- a single commodity, and will throw a program-terminating error if
+-- this is not possible.
+unifyMixedAmount :: MixedAmount -> Amount
+unifyMixedAmount mixedAmount = foldl combine (num 0) (amounts mixedAmount)
+  where
+    combine amount result =
+      if amountIsZero amount
+      then result
+      else if amountIsZero result
+        then amount
+        else if acommodity amount == acommodity result
+          then amount + result
+          else error' "Cannot calculate percentages for accounts with multiple commodities. (Hint: Try --cost, -V or similar flags.)"
+
+-- | Helper to calculate the percentage from two mixed. Keeps the sign of the first argument.
+-- Uses unifyMixedAmount to unify each argument and then divides them.
+perdivide :: MixedAmount -> MixedAmount -> MixedAmount
+perdivide a b =
+  let a' = unifyMixedAmount a
+      b' = unifyMixedAmount b
+  in if amountIsZero a' || amountIsZero b' || acommodity a' == acommodity b'
+    then mixed [per $ if aquantity b' == 0 then 0 else (aquantity a' / abs (aquantity b') * 100)]
+    else error' "Cannot calculate percentages if accounts have different commodities. (Hint: Try --cost, -V or similar flags.)"
+
+-- tests
+
 Right samplejournal2 =
   journalBalanceTransactions False
     nulljournal{
@@ -203,24 +252,22 @@ Right samplejournal2 =
       ]
     }
 
--- tests
-
 tests_BalanceReport = tests "BalanceReport" [
-  tests "balanceReport" $
-    let
-      (opts,journal) `gives` r = do
-        let (eitems, etotal) = r
-            (aitems, atotal) = balanceReport opts (queryFromOpts nulldate opts) journal
-            showw (acct,acct',indent,amt) = (acct, acct', indent, showMixedAmountDebug amt)
-        (map showw eitems) `is` (map showw aitems)
-        (showMixedAmountDebug etotal) `is` (showMixedAmountDebug atotal)
-      usd0 = usd 0
-    in [
-  
-     test "balanceReport with no args on null journal" $
+
+  let
+    (opts,journal) `gives` r = do
+      let (eitems, etotal) = r
+          (aitems, atotal) = balanceReport opts (queryFromOpts nulldate opts) journal
+          showw (acct,acct',indent,amt) = (acct, acct', indent, showMixedAmountDebug amt)
+      (map showw eitems) @?= (map showw aitems)
+      (showMixedAmountDebug etotal) @?= (showMixedAmountDebug atotal)
+  in
+    tests "balanceReport" [
+
+     test "no args, null journal" $
      (defreportopts, nulljournal) `gives` ([], Mixed [nullamt])
-  
-    ,test "balanceReport with no args on sample journal" $
+
+    ,test "no args, sample journal" $
      (defreportopts, samplejournal) `gives`
       ([
         ("assets","assets",0, mamountp' "$0.00")
@@ -235,45 +282,46 @@ tests_BalanceReport = tests "BalanceReport" [
        ,("income:gifts","gifts",1, mamountp' "$-1.00")
        ,("income:salary","salary",1, mamountp' "$-1.00")
        ],
-       Mixed [usd0])
-  
-    ,test "balanceReport with --depth=N" $
+       Mixed [usd 0])
+
+    ,test "with --depth=N" $
      (defreportopts{depth_=Just 1}, samplejournal) `gives`
       ([
        ("expenses",    "expenses",    0, mamountp'  "$2.00")
        ,("income",      "income",      0, mamountp' "$-2.00")
        ],
-       Mixed [usd0])
-  
-    ,test "balanceReport with depth:N" $
+       Mixed [usd 0])
+
+    ,test "with depth:N" $
      (defreportopts{query_="depth:1"}, samplejournal) `gives`
       ([
        ("expenses",    "expenses",    0, mamountp'  "$2.00")
        ,("income",      "income",      0, mamountp' "$-2.00")
        ],
-       Mixed [usd0])
-  
-    ,tests "balanceReport with a date or secondary date span" [
+       Mixed [usd 0])
+
+    ,test "with date:" $
      (defreportopts{query_="date:'in 2009'"}, samplejournal2) `gives`
       ([],
        Mixed [nullamt])
-     ,(defreportopts{query_="date2:'in 2009'"}, samplejournal2) `gives`
+
+    ,test "with date2:" $
+     (defreportopts{query_="date2:'in 2009'"}, samplejournal2) `gives`
       ([
         ("assets:bank:checking","assets:bank:checking",0,mamountp' "$1.00")
        ,("income:salary","income:salary",0,mamountp' "$-1.00")
        ],
-       Mixed [usd0])
-     ]
+       Mixed [usd 0])
 
-    ,test "balanceReport with desc:" $
+    ,test "with desc:" $
      (defreportopts{query_="desc:income"}, samplejournal) `gives`
       ([
         ("assets:bank:checking","assets:bank:checking",0,mamountp' "$1.00")
        ,("income:salary","income:salary",0, mamountp' "$-1.00")
        ],
-       Mixed [usd0])
-  
-    ,test "balanceReport with not:desc:" $
+       Mixed [usd 0])
+
+    ,test "with not:desc:" $
      (defreportopts{query_="not:desc:income"}, samplejournal) `gives`
       ([
         ("assets","assets",0, mamountp' "$-1.00")
@@ -284,23 +332,23 @@ tests_BalanceReport = tests "BalanceReport" [
        ,("expenses:supplies","supplies",1, mamountp' "$1.00")
        ,("income:gifts","income:gifts",0, mamountp' "$-1.00")
        ],
-       Mixed [usd0])
-  
-    ,test "balanceReport with period on a populated period" $
+       Mixed [usd 0])
+
+    ,test "with period on a populated period" $
       (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 1) (fromGregorian 2008 1 2)}, samplejournal) `gives`
        (
         [
          ("assets:bank:checking","assets:bank:checking",0, mamountp' "$1.00")
         ,("income:salary","income:salary",0, mamountp' "$-1.00")
         ],
-        Mixed [usd0])
-  
-     ,test "balanceReport with period on an unpopulated period" $
+        Mixed [usd 0])
+
+     ,test "with period on an unpopulated period" $
       (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 2) (fromGregorian 2008 1 3)}, samplejournal) `gives`
        ([],Mixed [nullamt])
-  
-  
-  
+
+
+
   {-
       ,test "accounts report with account pattern o" ~:
        defreportopts{patterns_=["o"]} `gives`
@@ -311,7 +359,7 @@ tests_BalanceReport = tests "BalanceReport" [
        ,"--------------------"
        ,"                 $-1"
        ]
-  
+
       ,test "accounts report with account pattern o and --depth 1" ~:
        defreportopts{patterns_=["o"],depth_=Just 1} `gives`
        ["                  $1  expenses"
@@ -319,7 +367,7 @@ tests_BalanceReport = tests "BalanceReport" [
        ,"--------------------"
        ,"                 $-1"
        ]
-  
+
       ,test "accounts report with account pattern a" ~:
        defreportopts{patterns_=["a"]} `gives`
        ["                 $-1  assets"
@@ -330,7 +378,7 @@ tests_BalanceReport = tests "BalanceReport" [
        ,"--------------------"
        ,"                 $-1"
        ]
-  
+
       ,test "accounts report with account pattern e" ~:
        defreportopts{patterns_=["e"]} `gives`
        ["                 $-1  assets"
@@ -346,7 +394,7 @@ tests_BalanceReport = tests "BalanceReport" [
        ,"--------------------"
        ,"                   0"
        ]
-  
+
       ,test "accounts report with unmatched parent of two matched subaccounts" ~:
        defreportopts{patterns_=["cash","saving"]} `gives`
        ["                 $-1  assets"
@@ -355,14 +403,14 @@ tests_BalanceReport = tests "BalanceReport" [
        ,"--------------------"
        ,"                 $-1"
        ]
-  
+
       ,test "accounts report with multi-part account name" ~:
        defreportopts{patterns_=["expenses:food"]} `gives`
        ["                  $1  expenses:food"
        ,"--------------------"
        ,"                  $1"
        ]
-  
+
       ,test "accounts report with negative account pattern" ~:
        defreportopts{patterns_=["not:assets"]} `gives`
        ["                  $2  expenses"
@@ -375,20 +423,20 @@ tests_BalanceReport = tests "BalanceReport" [
        ,"--------------------"
        ,"                  $1"
        ]
-  
+
       ,test "accounts report negative account pattern always matches full name" ~:
        defreportopts{patterns_=["not:e"]} `gives`
        ["--------------------"
        ,"                   0"
        ]
-  
+
       ,test "accounts report negative patterns affect totals" ~:
        defreportopts{patterns_=["expenses","not:food"]} `gives`
        ["                  $1  expenses:supplies"
        ,"--------------------"
        ,"                  $1"
        ]
-  
+
       ,test "accounts report with -E shows zero-balance accounts" ~:
        defreportopts{patterns_=["assets"],empty_=True} `gives`
        ["                 $-1  assets"
@@ -399,7 +447,7 @@ tests_BalanceReport = tests "BalanceReport" [
        ,"--------------------"
        ,"                 $-1"
        ]
-  
+
       ,test "accounts report with cost basis" $
          j <- (readJournal def Nothing $ unlines
                 [""
@@ -407,7 +455,7 @@ tests_BalanceReport = tests "BalanceReport" [
                 ,"  a:b          10h @ $50"
                 ,"  c:d                   "
                 ]) >>= either error' return
-         let j' = journalCanonicaliseAmounts $ journalConvertAmountsToCost j -- enable cost basis adjustment
+         let j' = journalCanonicaliseAmounts $ journalToCost j -- enable cost basis adjustment
          balanceReportAsText defreportopts (balanceReport defreportopts Any j') `is`
            ["                $500  a:b"
            ,"               $-500  c:d"
@@ -415,7 +463,7 @@ tests_BalanceReport = tests "BalanceReport" [
            ,"                   0"
            ]
   -}
-   ]
+     ]
 
  ]
 
