@@ -1,10 +1,16 @@
-{-# LANGUAGE RecordWildCards, DeriveDataTypeable, FlexibleInstances, TupleSections, OverloadedStrings #-}
 {-|
 
 Postings report, used by the register command.
 
 -}
 
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TupleSections #-}
+
 module Hledger.Reports.PostingsReport (
   PostingsReport,
   PostingsReportItem,
@@ -17,8 +23,8 @@ module Hledger.Reports.PostingsReport (
 where
 
 import Data.List
+import Data.List.Extra (nubSort)
 import Data.Maybe
-import Data.Ord (comparing)
 -- import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time.Calendar
@@ -26,7 +32,7 @@ import Safe (headMay, lastMay)
 
 import Hledger.Data
 import Hledger.Query
-import Hledger.Utils 
+import Hledger.Utils
 import Hledger.Reports.ReportOptions
 
 
@@ -52,37 +58,81 @@ type PostingsReportItem = (Maybe Day    -- The posting date, if this is the firs
                                         -- the running total/average.
                           )
 
+-- | A summary posting summarises the activity in one account within a report
+-- interval. It is kludgily represented by a regular Posting with no description,
+-- the interval's start date stored as the posting date, and the interval's end
+-- date attached with a tuple.
+type SummaryPosting = (Posting, Day)
+
 -- | Select postings from the journal and add running balance and other
 -- information to make a postings report. Used by eg hledger's register command.
 postingsReport :: ReportOpts -> Query -> Journal -> PostingsReport
-postingsReport opts q j = (totallabel, items)
+postingsReport ropts@ReportOpts{..} q j =
+  (totallabel, items)
     where
-      reportspan = adjustReportDates opts q j
-      whichdate = whichDateFromOpts opts
-      depth = queryDepth q
+      reportspan  = adjustReportDates ropts q j
+      whichdate   = whichDateFromOpts ropts
+      depth       = queryDepth q
+      styles      = journalCommodityStyles j
+      priceoracle = journalPriceOracle infer_value_ j
+      multiperiod = interval_ /= NoInterval
+      today       = fromMaybe (error' "postingsReport: could not pick a valuation date, ReportOpts today_ is unset") today_
 
       -- postings to be included in the report, and similarly-matched postings before the report start date
-      (precedingps, reportps) = matchedPostingsBeforeAndDuring opts q j reportspan
+      (precedingps, reportps) = matchedPostingsBeforeAndDuring ropts q j reportspan
 
-      -- postings or pseudo postings to be displayed
-      displayps | interval == NoInterval = map (,Nothing) reportps
-                | otherwise              = summarisePostingsByInterval interval whichdate depth showempty reportspan reportps
+      -- Postings, or summary postings with their subperiod's end date, to be displayed.
+      displayps :: [(Posting, Maybe Day)]
+        | multiperiod =
+            let summaryps = summarisePostingsByInterval interval_ whichdate depth showempty reportspan reportps
+            in [(pvalue p lastday, Just periodend) | (p, periodend) <- summaryps, let lastday = addDays (-1) periodend]
+        | otherwise =
+            [(pvalue p reportorjournallast, Nothing) | p <- reportps]
         where
-          interval = interval_ opts -- XXX
-          showempty = empty_ opts || average_ opts
+          showempty = empty_ || average_
+          -- We may be converting posting amounts to value, per hledger_options.m4.md "Effect of --value on reports".
+          pvalue p periodlast = maybe p (postingApplyValuation priceoracle styles periodlast mreportlast today multiperiod p) value_
+            where
+              mreportlast = reportPeriodLastDay ropts
+          reportorjournallast =
+            fromMaybe (error' "postingsReport: expected a non-empty journal") $ -- XXX shouldn't happen
+            reportPeriodOrJournalLastDay ropts j
 
-      -- posting report items ready for display
-      items = dbg1 "postingsReport items" $ postingsReportItems displayps (nullposting,Nothing) whichdate depth startbal runningcalc startnum
+      -- Posting report items ready for display.
+      items =
+        dbg1 "postingsReport items" $
+        postingsReportItems displayps (nullposting,Nothing) whichdate depth startbal runningcalc startnum
         where
-          historical = balancetype_ opts == HistoricalBalance
-          precedingsum = sumPostings precedingps
-          precedingavg | null precedingps = 0
-                       | otherwise        = divideMixedAmount (fromIntegral $ length precedingps) precedingsum
-          startbal | average_ opts = if historical then precedingavg else 0
-                   | otherwise     = if historical then precedingsum else 0
+          -- In historical mode we'll need a starting balance, which we
+          -- may be converting to value per hledger_options.m4.md "Effect
+          -- of --value on reports".
+          -- XXX balance report doesn't value starting balance.. should this ?
+          historical = balancetype_ == HistoricalBalance
+          startbal | average_  = if historical then bvalue precedingavg else 0
+                   | otherwise = if historical then bvalue precedingsum else 0
+            where
+              precedingsum = sumPostings precedingps
+              precedingavg | null precedingps = 0
+                           | otherwise        = divideMixedAmount (fromIntegral $ length precedingps) precedingsum
+              bvalue = maybe id (mixedAmountApplyValuation priceoracle styles daybeforereportstart Nothing today multiperiod) value_
+                  -- XXX constrain valuation type to AtDate daybeforereportstart here ?
+                where
+                  daybeforereportstart =
+                    maybe (error' "postingsReport: expected a non-empty journal") -- XXX shouldn't happen
+                    (addDays (-1))
+                    $ reportPeriodOrJournalStart ropts j
+
+          runningcalc = registerRunningCalculationFn ropts
           startnum = if historical then length precedingps + 1 else 1
-          runningcalc | average_ opts = \i avg amt -> divideMixedAmount (fromIntegral i) avg + amt - avg  -- running average
-                      | otherwise     = \_ bal amt -> bal + amt                                           -- running total
+
+-- | Based on the given report options, return a function that does the appropriate
+-- running calculation for the register report, ie a running average or running total.
+-- This function will take the item number, previous average/total, and new posting amount,
+-- and return the new average/total.
+registerRunningCalculationFn :: ReportOpts -> (Int -> MixedAmount -> MixedAmount -> MixedAmount)
+registerRunningCalculationFn ropts
+  | average_ ropts = \i avg amt -> avg + divideMixedAmount (fromIntegral i) (amt - avg)
+  | otherwise      = \_ bal amt -> bal + amt
 
 totallabel = "Total"
 
@@ -116,12 +166,13 @@ matchedPostingsBeforeAndDuring opts q j (DateSpan mstart mend) =
   where
     beforestartq = dbg1 "beforestartq" $ dateqtype $ DateSpan Nothing mstart
     beforeandduringps =
-      dbg1 "ps5" $ sortBy (comparing sortdate) $                               -- sort postings by date or date2
+      dbg1 "ps5" $ sortOn sortdate $                                           -- sort postings by date or date2
       dbg1 "ps4" $ (if invert_ opts then map negatePostingAmount else id) $    -- with --invert, invert amounts
       dbg1 "ps3" $ map (filterPostingAmount symq) $                            -- remove amount parts which the query's cur: terms would exclude
       dbg1 "ps2" $ (if related_ opts then concatMap relatedPostings else id) $ -- with -r, replace each with its sibling postings
       dbg1 "ps1" $ filter (beforeandduringq `matchesPosting`) $                -- filter postings by the query, with no start date or depth limit
-                  journalPostings $ journalSelectingAmountFromOpts opts j
+                  journalPostings $
+                  journalSelectingAmountFromOpts opts j    -- maybe convert to cost early, will be seen by amt:. XXX what about converting to value ?
       where
         beforeandduringq = dbg1 "beforeandduringq" $ And [depthless $ dateless q, beforeendq]
           where
@@ -136,9 +187,6 @@ matchedPostingsBeforeAndDuring opts q j (DateSpan mstart mend) =
       where
         dateq = dbg1 "dateq" $ filterQuery queryIsDateOrDate2 $ dbg1 "q" q  -- XXX confused by multiple date:/date2: ?
 
-negatePostingAmount :: Posting -> Posting
-negatePostingAmount p = p { pamount = negate $ pamount p }
-
 -- | Generate postings report line items from a list of postings or (with
 -- non-Nothing dates attached) summary postings.
 postingsReportItems :: [(Posting,Maybe Day)] -> (Posting,Maybe Day) -> WhichDate -> Int -> MixedAmount -> (Int -> MixedAmount -> MixedAmount -> MixedAmount) -> Int -> [PostingsReportItem]
@@ -172,21 +220,16 @@ mkpostingsReportItem showdate showdesc wd menddate p b =
 
 -- | Convert a list of postings into summary postings, one per interval,
 -- aggregated to the specified depth if any.
+-- Each summary posting will have a non-Nothing interval end date.
 summarisePostingsByInterval :: Interval -> WhichDate -> Int -> Bool -> DateSpan -> [Posting] -> [SummaryPosting]
 summarisePostingsByInterval interval wd depth showempty reportspan ps = concatMap summarisespan $ splitSpan interval reportspan
     where
       summarisespan s = summarisePostingsInDateSpan s wd depth showempty (postingsinspan s)
       postingsinspan s = filter (isPostingInDateSpan' wd s) ps
 
--- | A summary posting summarises the activity in one account within a report
--- interval. It is currently kludgily represented by a regular Posting with no
--- description, the interval's start date stored as the posting date, and the
--- interval's end date attached with a tuple.
-type SummaryPosting = (Posting, Maybe Day)
-
 -- | Given a date span (representing a report interval) and a list of
 -- postings within it, aggregate the postings into one summary posting per
--- account.
+-- account. Each summary posting will have a non-Nothing interval end date.
 --
 -- When a depth argument is present, postings to accounts of greater
 -- depth are also aggregated where possible. If the depth is 0, all
@@ -199,7 +242,7 @@ type SummaryPosting = (Posting, Maybe Day)
 summarisePostingsInDateSpan :: DateSpan -> WhichDate -> Int -> Bool -> [Posting] -> [SummaryPosting]
 summarisePostingsInDateSpan (DateSpan b e) wd depth showempty ps
     | null ps && (isNothing b || isNothing e) = []
-    | null ps && showempty = [(summaryp, Just e')]
+    | null ps && showempty = [(summaryp, e')]
     | otherwise = summarypes
     where
       postingdate = if wd == PrimaryDate then postingDate else postingDate2
@@ -210,8 +253,8 @@ summarisePostingsInDateSpan (DateSpan b e) wd depth showempty ps
                     | otherwise = ["..."]
       summaryps | depth > 0 = [summaryp{paccount=a,pamount=balance a} | a <- clippedanames]
                 | otherwise = [summaryp{paccount="...",pamount=sum $ map pamount ps}]
-      summarypes = map (, Just e') $ (if showempty then id else filter (not . isZeroMixedAmount . pamount)) summaryps
-      anames = sort $ nub $ map paccount ps
+      summarypes = map (, e') $ (if showempty then id else filter (not . mixedAmountLooksZero . pamount)) summaryps
+      anames = nubSort $ map paccount ps
       -- aggregate balances by account, like ledgerFromJournal, then do depth-clipping
       accts = accountsFromPostings ps
       balance a = maybe nullmixedamt bal $ lookupAccount a accts
@@ -219,27 +262,29 @@ summarisePostingsInDateSpan (DateSpan b e) wd depth showempty ps
           bal = if isclipped a then aibalance else aebalance
           isclipped a = accountNameLevel a >= depth
 
+negatePostingAmount :: Posting -> Posting
+negatePostingAmount p = p { pamount = negate $ pamount p }
+
+
 -- tests
 
 tests_PostingsReport = tests "PostingsReport" [
 
-   tests "postingsReport" $
-    let (query, journal) `gives` n = (length $ snd $ postingsReport defreportopts query journal) `is` n
-    in [
-     -- with the query specified explicitly
-      (Any, nulljournal) `gives` 0
-     ,(Any, samplejournal) `gives` 13
-     -- register --depth just clips account names
-     ,(Depth 2, samplejournal) `gives` 13
-     ,(And [Depth 1, StatusQ Cleared, Acct "expenses"], samplejournal) `gives` 2
-     ,(And [And [Depth 1, StatusQ Cleared], Acct "expenses"], samplejournal) `gives` 2
-  
-     -- with query and/or command-line options
-     ,(length $ snd $ postingsReport defreportopts Any samplejournal) `is` 13
-     ,(length $ snd $ postingsReport defreportopts{interval_=Months 1} Any samplejournal) `is` 11
-     ,(length $ snd $ postingsReport defreportopts{interval_=Months 1, empty_=True} Any samplejournal) `is` 20
-     ,(length $ snd $ postingsReport defreportopts (Acct "assets:bank:checking") samplejournal) `is` 5
-  
+   test "postingsReport" $ do
+    let (query, journal) `gives` n = (length $ snd $ postingsReport defreportopts query journal) @?= n
+    -- with the query specified explicitly
+    (Any, nulljournal) `gives` 0
+    (Any, samplejournal) `gives` 13
+    -- register --depth just clips account names
+    (Depth 2, samplejournal) `gives` 13
+    (And [Depth 1, StatusQ Cleared, Acct "expenses"], samplejournal) `gives` 2
+    (And [And [Depth 1, StatusQ Cleared], Acct "expenses"], samplejournal) `gives` 2
+    -- with query and/or command-line options
+    (length $ snd $ postingsReport defreportopts Any samplejournal) @?= 13
+    (length $ snd $ postingsReport defreportopts{interval_=Months 1} Any samplejournal) @?= 11
+    (length $ snd $ postingsReport defreportopts{interval_=Months 1, empty_=True} Any samplejournal) @?= 20
+    (length $ snd $ postingsReport defreportopts (Acct "assets:bank:checking") samplejournal) @?= 5
+
      -- (defreportopts, And [Acct "a a", Acct "'b"], samplejournal2) `gives` 0
      -- [(Just (parsedate "2008-01-01","income"),assets:bank:checking             $1,$1)
      -- ,(Nothing,income:salary                   $-1,0)
@@ -251,7 +296,7 @@ tests_PostingsReport = tests "PostingsReport" [
      -- ,(Nothing,expenses:supplies                $1,$2)
      -- ,(Nothing,assets:cash                     $-2,0)
      -- ,(Just (2008-12-31,"pay off"),liabilities:debts                $1,$1)
-     -- ,(Nothing,assets:bank:checking            $-1,0)    
+     -- ,(Nothing,assets:bank:checking            $-1,0)
 
     {-
         let opts = defreportopts
@@ -268,7 +313,7 @@ tests_PostingsReport = tests "PostingsReport" [
          ,"2008/12/31 pay off              liabilities:debts                $1           $1"
          ,"                                assets:bank:checking            $-1            0"
          ]
-    
+
       ,"postings report with cleared option" ~:
        do
         let opts = defreportopts{cleared_=True}
@@ -280,7 +325,7 @@ tests_PostingsReport = tests "PostingsReport" [
          ,"2008/12/31 pay off              liabilities:debts                $1           $1"
          ,"                                assets:bank:checking            $-1            0"
          ]
-    
+
       ,"postings report with uncleared option" ~:
        do
         let opts = defreportopts{uncleared_=True}
@@ -293,7 +338,7 @@ tests_PostingsReport = tests "PostingsReport" [
          ,"2008/06/02 save                 assets:bank:saving               $1           $1"
          ,"                                assets:bank:checking            $-1            0"
          ]
-    
+
       ,"postings report sorts by date" ~:
        do
         j <- readJournal' $ unlines
@@ -307,7 +352,7 @@ tests_PostingsReport = tests "PostingsReport" [
             ]
         let opts = defreportopts
         registerdates (postingsReportAsText opts $ postingsReport opts (queryFromOpts date1 opts) j) `is` ["2008/01/01","2008/02/02"]
-    
+
       ,"postings report with account pattern" ~:
        do
         j <- samplejournal
@@ -315,7 +360,7 @@ tests_PostingsReport = tests "PostingsReport" [
         (postingsReportAsText opts $ postingsReport opts (queryFromOpts date1 opts) j) `is` unlines
          ["2008/06/03 eat & shop           assets:cash                     $-2          $-2"
          ]
-    
+
       ,"postings report with account pattern, case insensitive" ~:
        do
         j <- samplejournal
@@ -323,7 +368,7 @@ tests_PostingsReport = tests "PostingsReport" [
         (postingsReportAsText opts $ postingsReport opts (queryFromOpts date1 opts) j) `is` unlines
          ["2008/06/03 eat & shop           assets:cash                     $-2          $-2"
          ]
-    
+
       ,"postings report with display expression" ~:
        do
         j <- samplejournal
@@ -335,7 +380,7 @@ tests_PostingsReport = tests "PostingsReport" [
         "d=[2008/6/2]"  `gives` ["2008/06/02"]
         "d>=[2008/6/2]" `gives` ["2008/06/02","2008/06/03","2008/12/31"]
         "d>[2008/6/2]"  `gives` ["2008/06/03","2008/12/31"]
-    
+
       ,"postings report with period expression" ~:
        do
         j <- samplejournal
@@ -363,9 +408,9 @@ tests_PostingsReport = tests "PostingsReport" [
         registerdates (postingsReportAsText opts $ postingsReport opts (queryFromOpts date1 opts) j) `is` ["2008/01/01","2008/04/01","2008/10/01"]
         let opts = defreportopts{period_=maybePeriod date1 "quarterly",empty_=True}
         registerdates (postingsReportAsText opts $ postingsReport opts (queryFromOpts date1 opts) j) `is` ["2008/01/01","2008/04/01","2008/07/01","2008/10/01"]
-    
+
       ]
-    
+
       , "postings report with depth arg" ~:
        do
         j <- samplejournal
@@ -383,16 +428,12 @@ tests_PostingsReport = tests "PostingsReport" [
          ,"2008/12/31 pay off              liabilities:debts                $1           $1"
          ,"                                assets:bank                     $-1            0"
          ]
-    
-    -}
-    ]
 
-  ,tests "summarisePostingsByInterval" [
-    tests "summarisePostingsByInterval" [
-      summarisePostingsByInterval (Quarters 1) PrimaryDate 99999 False (DateSpan Nothing Nothing) [] `is` []
-      ]
-   ]
-  
+    -}
+
+  ,test "summarisePostingsByInterval" $
+    summarisePostingsByInterval (Quarters 1) PrimaryDate 99999 False (DateSpan Nothing Nothing) [] @?= []
+
   -- ,tests_summarisePostingsInDateSpan = [
     --  "summarisePostingsInDateSpan" ~: do
     --   let gives (b,e,depth,showempty,ps) =
@@ -428,5 +469,5 @@ tests_PostingsReport = tests "PostingsReport" [
     --    [
     --     nullposting{lpdate=parsedate "2008/01/01",lpdescription="- 2008/12/31",lpaccount="",lpamount=Mixed [usd 15]}
     --    ]
-  
+
  ]
