@@ -1,5 +1,33 @@
 {-# LANGUAGE FlexibleContexts, TypeFamilies #-}
--- | Debugging helpers
+{- | Debugging helpers.
+
+You can enable increasingly verbose debug output by adding --debug [1-9]
+to a hledger command line. --debug with no argument means --debug 1.
+This is implemented by calling dbgN or similar helpers, defined below.
+These calls can be found throughout hledger code; they have been added
+organically where it seemed likely they would be needed again.
+The choice of debug level has not been very systematic.
+202006 Here's a start at some guidelines, not yet applied project-wide:
+
+Debug level:  What to show:
+------------  ---------------------------------------------------------
+0             normal command output only (no warnings, eg)
+1 (--debug)   useful warnings, most common troubleshooting info, eg valuation
+2             common troubleshooting info, more detail
+3             report options selection
+4             report generation
+5             report generation, more detail
+6             command line parsing
+7             input file reading
+8             input file reading, more detail
+9             any other rarely needed / more in-depth info
+
+Tip: when debugging with GHCI, the first run after loading Debug.hs sets the
+debug level. If you need to change it, you must touch Debug.hs, :reload in GHCI,
+then run the command with a new --debug value. Or, often it's more convenient
+to add a temporary dbg0 and :reload (dbg0 always prints).
+
+-}
 
 -- more:
 -- http://hackage.haskell.org/packages/archive/TraceUtils/0.1.0.2/doc/html/Debug-TraceUtils.html
@@ -12,10 +40,12 @@ module Hledger.Utils.Debug (
   ,pshow
   ,ptrace
   ,traceWith
+  ,traceAt
+  ,traceAtWith
   ,debugLevel
   ,ptraceAt
+  ,ptraceAtWith
   ,dbg0
-  ,dbgExit
   ,dbg1
   ,dbg2
   ,dbg3
@@ -25,6 +55,17 @@ module Hledger.Utils.Debug (
   ,dbg7
   ,dbg8
   ,dbg9
+  ,dbg0With
+  ,dbg1With
+  ,dbg2With
+  ,dbg3With
+  ,dbg4With
+  ,dbg5With
+  ,dbg6With
+  ,dbg7With
+  ,dbg8With
+  ,dbg9With
+  ,dbgExit
   ,ptraceAtIO
   ,dbg0IO
   ,dbg1IO
@@ -70,8 +111,9 @@ pshow = ppShow
 ptrace :: Show a => a -> a
 ptrace = traceWith pshow
 
--- | Trace (print to stderr) a showable value using a custom show function.
-traceWith :: (a -> String) -> a -> a
+-- | Like traceShowId, but uses a custom show function to render the value.
+-- traceShowIdWith was too much of a mouthful.
+traceWith :: Show a => (a -> String) -> a -> a
 traceWith f a = trace (f a) a
 
 -- | Global debug level, which controls the verbosity of debug output
@@ -83,7 +125,7 @@ traceWith f a = trace (f a) a
 -- touch and reload this module to see the effect of a new --debug option.
 -- After command-line processing, it is also available as the @debug_@
 -- field of 'Hledger.Cli.CliOptions.CliOpts'.
--- {-# OPTIONS_GHC -fno-cse #-} 
+-- {-# OPTIONS_GHC -fno-cse #-}
 -- {-# NOINLINE debugLevel #-}
 debugLevel :: Int
 debugLevel = case snd $ break (=="--debug") args of
@@ -97,6 +139,18 @@ debugLevel = case snd $ break (=="--debug") args of
     where
       args = unsafePerformIO getArgs
 
+-- | Trace (print to stderr) a string if the global debug level is at
+-- or above the specified level. At level 0, always prints. Otherwise,
+-- uses unsafePerformIO.
+traceAt :: Int -> String -> a -> a
+traceAt level
+    | level > 0 && debugLevel < level = flip const
+    | otherwise = trace
+
+-- | Trace (print to stderr) a showable value using a custom show function.
+traceAtWith :: (a -> String) -> a -> a
+traceAtWith f a = trace (f a) a
+
 -- | Pretty-print a label and a showable value to the console
 -- if the global debug level is at or above the specified level.
 -- At level 0, always prints. Otherwise, uses unsafePerformIO.
@@ -111,16 +165,25 @@ ptraceAt level
                                   | otherwise     = ls
                           in trace (s++":"++nlorspace++intercalate "\n" ls') a
 
--- | Pretty-print a message and the showable value to the console, then return it.
+-- | Like ptraceAt, but takes a custom show function instead of a label.
+ptraceAtWith :: Show a => Int -> (a -> String) -> a -> a
+ptraceAtWith level f
+    | level > 0 && debugLevel < level = id
+    | otherwise = \a -> let p = f a
+                            -- ls = lines p
+                            -- nlorspace | length ls > 1 = "\n"
+                            --           | otherwise     = " " ++ take (10 - length s) (repeat ' ')
+                            -- ls' | length ls > 1 = map (" "++) ls
+                            --     | otherwise     = ls
+                        -- in trace (s++":"++nlorspace++intercalate "\n" ls') a
+                        in trace p a
+
+-- "dbg" would clash with megaparsec.
+-- | Pretty-print a label and the showable value to the console, then return it.
 dbg0 :: Show a => String -> a -> a
 dbg0 = ptraceAt 0
--- "dbg" would clash with megaparsec
 
--- | Like dbg0, but also exit the program. Uses unsafePerformIO.
-dbgExit :: Show a => String -> a -> a
-dbgExit msg = const (unsafePerformIO exitFailure) . dbg0 msg
-
--- | Pretty-print a message and the showable value to the console when the global debug level is >= 1, then return it.
+-- | Pretty-print a label and the showable value to the console when the global debug level is >= 1, then return it.
 -- Uses unsafePerformIO.
 dbg1 :: Show a => String -> a -> a
 dbg1 = ptraceAt 1
@@ -149,8 +212,43 @@ dbg8 = ptraceAt 8
 dbg9 :: Show a => String -> a -> a
 dbg9 = ptraceAt 9
 
--- | Like ptraceAt, but convenient to insert in an IO monad (plus
--- convenience aliases).
+-- | Like dbg0, but takes a custom show function instead of a label.
+dbg0With :: Show a => (a -> String) -> a -> a
+dbg0With = ptraceAtWith 0
+
+dbg1With :: Show a => (a -> String) -> a -> a
+dbg1With = ptraceAtWith 1
+
+dbg2With :: Show a => (a -> String) -> a -> a
+dbg2With = ptraceAtWith 2
+
+dbg3With :: Show a => (a -> String) -> a -> a
+dbg3With = ptraceAtWith 3
+
+dbg4With :: Show a => (a -> String) -> a -> a
+dbg4With = ptraceAtWith 4
+
+dbg5With :: Show a => (a -> String) -> a -> a
+dbg5With = ptraceAtWith 5
+
+dbg6With :: Show a => (a -> String) -> a -> a
+dbg6With = ptraceAtWith 6
+
+dbg7With :: Show a => (a -> String) -> a -> a
+dbg7With = ptraceAtWith 7
+
+dbg8With :: Show a => (a -> String) -> a -> a
+dbg8With = ptraceAtWith 8
+
+dbg9With :: Show a => (a -> String) -> a -> a
+dbg9With = ptraceAtWith 9
+
+-- | Like dbg0, but also exit the program. Uses unsafePerformIO.
+dbgExit :: Show a => String -> a -> a
+dbgExit msg = const (unsafePerformIO exitFailure) . dbg0 msg
+
+-- | Like ptraceAt, but convenient to insert in an IO monad and
+-- enforces monadic sequencing (plus convenience aliases).
 -- XXX These have a bug; they should use
 -- traceIO, not trace, otherwise GHC can occasionally over-optimise
 -- (cf lpaste a few days ago where it killed/blocked a child thread).
@@ -191,12 +289,12 @@ dbg8IO = ptraceAtIO 8
 dbg9IO :: (MonadIO m, Show a) => String -> a -> m ()
 dbg9IO = ptraceAtIO 9
 
--- | Log a message and a pretty-printed showable value to ./debug.log, then return it.
+-- | Log a label and a pretty-printed showable value to ./debug.log, then return it.
 -- Can fail, see plogAt.
 plog :: Show a => String -> a -> a
 plog = plogAt 0
 
--- | Log a message and a pretty-printed showable value to ./debug.log, 
+-- | Log a label and a pretty-printed showable value to ./debug.log,
 -- if the global debug level is at or above the specified level.
 -- At level 0, always logs. Otherwise, uses unsafePerformIO.
 -- Tends to fail if called more than once, at least when built with -threaded
@@ -204,7 +302,7 @@ plog = plogAt 0
 plogAt :: Show a => Int -> String -> a -> a
 plogAt lvl
     | lvl > 0 && debugLevel < lvl = flip const
-    | otherwise = \s a -> 
+    | otherwise = \s a ->
         let p = ppShow a
             ls = lines p
             nlorspace | length ls > 1 = "\n"
