@@ -14,7 +14,7 @@ module Hledger.Utils.Text
   textUnbracket,
  -- -- quoting
   quoteIfSpaced,
- -- quoteIfNeeded,
+  textQuoteIfNeeded,
  -- singleQuoteIfNeeded,
  -- -- quotechars,
  -- -- whitespacechars,
@@ -30,7 +30,7 @@ module Hledger.Utils.Text
   textstrip,
   textlstrip,
   textrstrip,
- -- chomp,
+  textchomp,
  -- elideLeft,
   textElideRight,
  -- formatString,
@@ -90,9 +90,9 @@ textlstrip = T.dropWhile (`elem` (" \t" :: String)) :: Text -> Text -- XXX isSpa
 textrstrip = T.reverse . textlstrip . T.reverse
 textrstrip :: Text -> Text
 
--- -- | Remove trailing newlines/carriage returns.
--- chomp :: String -> String
--- chomp = reverse . dropWhile (`elem` "\r\n") . reverse
+-- | Remove trailing newlines/carriage returns (and other whitespace).
+textchomp :: Text -> Text
+textchomp = T.stripEnd
 
 -- stripbrackets :: String -> String
 -- stripbrackets = dropWhile (`elem` "([") . reverse . dropWhile (`elem` "])") . reverse :: String -> String
@@ -127,7 +127,7 @@ textElideRight width t =
 quoteIfSpaced :: T.Text -> T.Text
 quoteIfSpaced s | isSingleQuoted s || isDoubleQuoted s = s
                 | not $ any (`elem` (T.unpack s)) whitespacechars = s
-                | otherwise = quoteIfNeeded s
+                | otherwise = textQuoteIfNeeded s
 
 -- -- | Wrap a string in double quotes, and \-prefix any embedded single
 -- -- quotes, if it contains whitespace and is not already single- or
@@ -139,9 +139,9 @@ quoteIfSpaced s | isSingleQuoted s || isDoubleQuoted s = s
 
 -- -- | Double-quote this string if it contains whitespace, single quotes
 -- -- or double-quotes, escaping the quotes as needed.
-quoteIfNeeded :: T.Text -> T.Text
-quoteIfNeeded s | any (`elem` T.unpack s) (quotechars++whitespacechars) = "\"" <> escapeDoubleQuotes s <> "\""
-                | otherwise = s
+textQuoteIfNeeded :: T.Text -> T.Text
+textQuoteIfNeeded s | any (`elem` T.unpack s) (quotechars++whitespacechars) = "\"" <> escapeDoubleQuotes s <> "\""
+                    | otherwise = s
 
 -- -- | Single-quote this string if it contains whitespace or double-quotes.
 -- -- No good for strings containing single quotes.
@@ -421,13 +421,12 @@ textWidth s = maximum $ map (T.foldr (\a b -> charWidth a + b) 0) $ T.lines s
 
 
 tests_Text = tests "Text" [
-   tests "quoteIfSpaced" [
-     quoteIfSpaced "a'a" `is` "a'a"
-    ,quoteIfSpaced "a\"a" `is` "a\"a"              
-    ,quoteIfSpaced "a a" `is` "\"a a\""               
-    ,quoteIfSpaced "mimi's cafe" `is` "\"mimi's cafe\""       
-    ,quoteIfSpaced "\"alex\" cafe" `is` "\"\\\"alex\\\" cafe\""     
-    ,quoteIfSpaced "le'shan's cafe" `is` "\"le'shan's cafe\""    
-    ,quoteIfSpaced "\"be'any's\" cafe" `is` "\"\\\"be'any's\\\" cafe\"" 
-    ] 
+   test "quoteIfSpaced" $ do
+     quoteIfSpaced "a'a" @?= "a'a"
+     quoteIfSpaced "a\"a" @?= "a\"a"
+     quoteIfSpaced "a a" @?= "\"a a\""
+     quoteIfSpaced "mimi's cafe" @?= "\"mimi's cafe\""
+     quoteIfSpaced "\"alex\" cafe" @?= "\"\\\"alex\\\" cafe\""
+     quoteIfSpaced "le'shan's cafe" @?= "\"le'shan's cafe\""
+     quoteIfSpaced "\"be'any's\" cafe" @?= "\"\\\"be'any's\\\" cafe\""
   ]
